;;; -*- Mode: Lisp; Package: FORM -*-
(in-package "FORM")  ;; creates package for abstract syntax. 

(in-package "FORM")  ;; enters package for generated code.  

(use-package '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP"))


(export '())

(DEFPARAMETER FORM-KEYWORD-LIST
              '(SBST::ENDCASE SBST::|;| SBST::|:| SBST::OF SBST::CASE
               SBST::ENDIF SBST::ELSE SBST::THEN SBST::IF SBST::@ SBST::*
               SBST::+ SBST::> SBST::|,| SBST::< SBST::|,| SBST::|(| SBST::|)|)) 
(DEFPARAMETER FORM-SINGLE-CHAR-OP-LIST
              '(#\@ #\, #\; #\] #\[ #\= #\+ #\- #\* #\^ #\) #\()) 
(DEFPARAMETER FORM-MULTI-CHAR-OP-LIST
              '((#\> . LEX->) (#\< . LEX-<) (#\: . |LEX-:|) (#\! . LEX-!))) 
(DEFPARAMETER FORM-ALL-OPERATORS-LIST
              '(SBST::@ SBST::! SBST::!! SBST::|,| SBST::|:| SBST::|;|
               SBST::|:=| SBST::<> SBST::>= SBST::<= SBST::> SBST::< SBST::]
               SBST::[ SBST::= SBST::+ SBST::- SBST::* SBST::^ SBST::|)|
               SBST::|(|)) 
(DEFPARAMETER FORM-NEW-LINE-COMMENT-CHAR NIL) 
(DEFPARAMETER FORM-OPEN-COMMENT-CHAR NIL) 
(DEFPARAMETER FORM-CLOSE-COMMENT-CHAR NIL) 
(DEFPARAMETER FORM-ESCAPE-CHAR NIL) 
(DEFPARAMETER FORM-CASE-SENSITIVE NIL) 
(DEFPARAMETER FORM-STRING-CHAR #\") 
(DEFPARAMETER FORM-KEYWORD-CHAR NIL) 
(DEFPARAMETER FORM-LITERAL-CHAR NIL) 
(DEFPARAMETER FORM-RESTRICTED-CHARS
              (REDUCE #'(LAMBDA (R
                                 S)
                          (UNION R S :TEST #'CHAR=))
                      (LIST FORM-SINGLE-CHAR-OP-LIST
                            (IF FORM-NEW-LINE-COMMENT-CHAR
                                (LIST FORM-NEW-LINE-COMMENT-CHAR))
                            (IF FORM-OPEN-COMMENT-CHAR
                                (LIST FORM-OPEN-COMMENT-CHAR))
                            (IF FORM-CLOSE-COMMENT-CHAR
                                (LIST FORM-CLOSE-COMMENT-CHAR))
                            (IF FORM-ESCAPE-CHAR (LIST FORM-ESCAPE-CHAR))
                            (IF FORM-STRING-CHAR (LIST FORM-STRING-CHAR))
                            (IF FORM-KEYWORD-CHAR (LIST FORM-KEYWORD-CHAR))
                            (IF FORM-LITERAL-CHAR (LIST FORM-LITERAL-CHAR))))) 
(DEFVAR *FORM-KEYWORD-TABLE* NIL) 
(DEFUN INIT-LEXER-AUX (LEXSTREAM)
  (INIT-LEXICAL-READTABLE LEXSTREAM :SINGLE-CHAR-OP-LIST
                          FORM-SINGLE-CHAR-OP-LIST :NEW-LINE-COMMENT-CHAR
                          FORM-NEW-LINE-COMMENT-CHAR :OPEN-COMMENT-CHAR
                          FORM-OPEN-COMMENT-CHAR :ESCAPE-CHAR FORM-ESCAPE-CHAR
                          :MULTI-CHAR-OP-LIST FORM-MULTI-CHAR-OP-LIST)) 

(DEFUN INIT-LEXER (LEXSTREAM)
  (INIT-LEXER-AUX LEXSTREAM)
  (IF FORM-STRING-CHAR
      (LEXICAL-MAKE-MACRO LEXSTREAM FORM-STRING-CHAR #'READ-SB-STRING))
  (IF FORM-KEYWORD-CHAR
      (LEXICAL-MAKE-MACRO LEXSTREAM FORM-KEYWORD-CHAR #'READ-KEYWORD-STRING))
  (IF FORM-LITERAL-CHAR
      (LEXICAL-MAKE-MACRO LEXSTREAM FORM-LITERAL-CHAR #'READ-LITERAL))) 

(DEFUN LEX-! (STREAM SYMBOL)
  (DECLARE (IGNORE SYMBOL))
  (LET (HOLDCHAR)
    (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF))
    (IF (AND FORM-ESCAPE-CHAR (CHAR= HOLDCHAR FORM-ESCAPE-CHAR))
        (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF)))
    (COND ((CHAR= HOLDCHAR #\!)
           'SBST::!!)
          (T
           (LEXICAL-UNREAD-CHAR STREAM)
           'SBST::!)))) 
(DEFUN |LEX-:| (STREAM SYMBOL)
  (DECLARE (IGNORE SYMBOL))
  (LET (HOLDCHAR)
    (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF))
    (IF (AND FORM-ESCAPE-CHAR (CHAR= HOLDCHAR FORM-ESCAPE-CHAR))
        (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF)))
    (COND ((CHAR= HOLDCHAR #\=)
           'SBST::|:=|)
          (T
           (LEXICAL-UNREAD-CHAR STREAM)
           'SBST::|:|)))) 
(DEFUN LEX-< (STREAM SYMBOL)
  (DECLARE (IGNORE SYMBOL))
  (LET (HOLDCHAR)
    (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF))
    (IF (AND FORM-ESCAPE-CHAR (CHAR= HOLDCHAR FORM-ESCAPE-CHAR))
        (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF)))
    (COND ((CHAR= HOLDCHAR #\>)
           'SBST::<>)
          ((CHAR= HOLDCHAR #\=)
           'SBST::<=)
          (T
           (LEXICAL-UNREAD-CHAR STREAM)
           'SBST::<)))) 
(DEFUN LEX-> (STREAM SYMBOL)
  (DECLARE (IGNORE SYMBOL))
  (LET (HOLDCHAR)
    (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF))
    (IF (AND FORM-ESCAPE-CHAR (CHAR= HOLDCHAR FORM-ESCAPE-CHAR))
        (SETF HOLDCHAR (LEXICAL-READ-CHAR STREAM :EOF)))
    (COND ((CHAR= HOLDCHAR #\=)
           'SBST::>=)
          (T
           (LEXICAL-UNREAD-CHAR STREAM)
           'SBST::>)))) 
