;;; -*- Mode: Lisp; Package: FORM -*-
(in-package "FORM")  ;; creates package for abstract syntax. 

(in-package "FORM")  ;; enters package for generated code.  

(use-package '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP"))


(export '( FORM-PARSE ))

(DEFPARAMETER FORM-ABS-SYN-PACKAGE (FIND-PACKAGE "FORM")) 

(DEFUN FORM-PARSE (&KEY (NT 'EXP) ERROR-THRESHOLD ASK-ABOUT-BAD-TOKENS (RETURN-ERRORS NIL)
  (STREAM NIL STREAMP) STRING FILE (EXHAUST-STREAM NIL))
  (COND (STREAM)
        (STRING
         (SETF STREAM (MAKE-STRING-INPUT-STREAM STRING)))
        (FILE
         (SETF STREAM (OPEN FILE)))
        (T
         (ERROR
          "Must provide an input means -- either :stream, ~
			:string, or :file.")))
  (SETQ *FORM-KEYWORD-TABLE* (INIT-KEYWORD-TABLE FORM-KEYWORD-LIST))
  (LET ((*PARSER-ERROR* NIL)
        (*PARSER-ERROR-COUNT* 0)
        (*PARSER-RETURN-ERRORS* RETURN-ERRORS)
        (*PARSER-ABORT-THRESHOLD* ERROR-THRESHOLD)
        (*ASK-ABOUT-BAD-TOKENS* ASK-ABOUT-BAD-TOKENS)
        (*LEXICAL-STREAM* (OPEN-LEXICAL-STREAM (OPEN-PLACESTREAM STREAM)))
        (*HOLD-A1* NIL)
        (*HOLD-A2* NIL)
        (*HOLD-B1* NIL)
        (*HOLD-B2* NIL)
        (*ABS-SYN-PACKAGE* FORM-ABS-SYN-PACKAGE)
        (*READER-FUN* #'READER)
        (*APPLY-LEX-TERM-CONSTR-FUN* #'APPLY-LEXICAL-TERMINAL-CONSTRUCTOR)
        (*KEYWORD-TABLE* *FORM-KEYWORD-TABLE*)
        (*CLOSE-COMMENT-CHAR* FORM-CLOSE-COMMENT-CHAR)
        (*CASE-SENSITIVE* FORM-CASE-SENSITIVE))
    (INIT-LEXER *LEXICAL-STREAM*)
    (MULTIPLE-VALUE-BIND (ABS-SYNTAX ERROR-STRING ERROR-ARGS ERROR-PLACE) (UNWIND-PROTECT (CATCH
                      'PARSER-ABORT-CATCH
                      (PROG1
                        (CASE NT
                          (EXP (!EXP! 0))
                          (T (ERROR "Unknown nonterminal ~A." NT)))
                        (WHEN (OR FILE STRING EXHAUST-STREAM)
                          (CLET* (((TOKEN IGNORE PLACE) (PEEK-FIRST)))
                                 (UNLESS (OR (EQ TOKEN :EOF) (EQ TOKEN 'EOF))
                                   (DO-SYNTAX-ERROR
                                    "There is garbage at the end of your ~
			      file or string:~%~A"
                                    PLACE))))))
                     (UNLESS
                      STREAMP
                      (CLOSE STREAM)))
      (IF RETURN-ERRORS
          (VALUES ABS-SYNTAX *PARSER-ERROR* ERROR-PLACE ERROR-STRING ERROR-ARGS)
          (VALUES ABS-SYNTAX *PARSER-ERROR*))))) 


(DEFUN !EXP! (RBP &OPTIONAL (BRACKET-LIST (EMPTY-BRACKET-LIST)))
  (LET (MTEMP
        V0
        V1
        V2
        V3
        V4
        V5
        V6
        V7)
    (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6 V7)
    (STACK-BRACKETS SBST::|(| EXP)
    (INITIALS
     ((LA-MATCH ((SBST::CASE)))
      (PROGN
        (LAM ((SBST::CASE EPSILON)) (GOBBLE-TOKEN))
        (CODE-TO-SLOT V2 (!EXP! 0))
        (LAM ((SBST::OF EPSILON)) (GOBBLE-TOKEN))
        (PLUS-PARSE
         ((SBST::!ID! SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE SBST::IF
           SBST::< SBST::!NUMBER! SBST::!ID! SBST::|:|)
          (SBST::!NUMBER! SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE SBST::IF
           SBST::< SBST::!NUMBER! SBST::!ID! SBST::|:|)
          (SBST::< SBST::> SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::IF SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::CASE SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::|(| SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!))
         (PROGN
           (CODE-TO-SLOT V5 (!EXP! 0))
           (LAM ((SBST::|:| EPSILON)) (GOBBLE-TOKEN))
           (CODE-TO-SLOT V6 (!EXP! 0))
           (LAM ((SBST::|;| EPSILON)) (GOBBLE-TOKEN))
           (AS-TO-SLOT V7 (MAKE-SB-TERM 'FORM::ENTRY (LIST V5 V6)))
           (VALUE-TO-SLOT V4 V7))
         V3 V4)
        (LAM ((SBST::ENDCASE EPSILON)) (GOBBLE-TOKEN))
        (VALUE-TO-SLOT V0 9)
        (AS-TO-SLOT V1
                    (MAKE-SB-TERM 'CASE
                                  (LIST V2
                                        (MAKE-SB-TERM 'FORM::CHOICES
                                                      (DS-TEMP-REPT
                                                       (CK-REPT-REF V3))))))))
     ((LA-MATCH ((SBST::IF)))
      (PROGN
        (LAM ((SBST::IF EPSILON)) (GOBBLE-TOKEN))
        (CODE-TO-SLOT V2 (!EXP! 0))
        (LAM ((SBST::THEN EPSILON)) (GOBBLE-TOKEN))
        (CODE-TO-SLOT V3 (!EXP! 0))
        (OPT-PARSE
         ((SBST::ELSE SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!))
         (PROGN
           (LAM ((SBST::ELSE EPSILON)) (GOBBLE-TOKEN))
           (CODE-TO-SLOT V5 (!EXP! 0)))
         V4)
        (LAM ((SBST::ENDIF EPSILON)) (GOBBLE-TOKEN))
        (VALUE-TO-SLOT V0 8)
        (AS-TO-SLOT V1
                    (IF (= V4 0) (MAKE-SB-TERM 'IF (LIST V2 V3))
                        (MAKE-SB-TERM 'FORM::IF-ELSE (LIST V2 V3 V5))))))
     ((LA-MATCH ((SBST::<)))
      (PROGN
        (LAM ((SBST::< EPSILON)) (GOBBLE-TOKEN))
        (DOUBLESTAR-PARSE
         ((SBST::|(| SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::CASE SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::IF SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::< SBST::> SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::!NUMBER! EPSILON SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE
           SBST::IF SBST::< SBST::!NUMBER! SBST::!ID!)
          (SBST::!ID! EPSILON SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE
           SBST::IF SBST::< SBST::!NUMBER! SBST::!ID!))
         (PROGN
           (CODE-TO-SLOT V3 (!EXP! 0))
           NIL) V2 V3 'SBST::|,|)
        (LAM ((SBST::> EPSILON)) (GOBBLE-TOKEN))
        (VALUE-TO-SLOT V0 2)
        (AS-TO-SLOT V1
                    (MAKE-SB-TERM 'FORM::TUPLE (DS-TEMP-REPT (CK-REPT-REF V2))))))
     ((LA-MATCH ((SBST::!ID! SBST::|(|)))
      (PROGN
        (LAM ((SBST::!ID! EPSILON)) (GOBBLE-TO-SLOT V2))
        (LAM ((SBST::|(| EPSILON)) (GOBBLE-TOKEN))
        (DOUBLESTAR-PARSE
         ((SBST::|(| SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::CASE SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::IF SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::< SBST::> SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
           SBST::!ID!)
          (SBST::!NUMBER! EPSILON SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE
           SBST::IF SBST::< SBST::!NUMBER! SBST::!ID!)
          (SBST::!ID! EPSILON SBST::+ SBST::* SBST::@ SBST::|(| SBST::CASE
           SBST::IF SBST::< SBST::!NUMBER! SBST::!ID!))
         (PROGN
           (CODE-TO-SLOT V4 (!EXP! 0))
           NIL) V3 V4 'SBST::|,|)
        (LAM ((SBST::|)| EPSILON)) (GOBBLE-TOKEN))
        (VALUE-TO-SLOT V0 3)
        (AS-TO-SLOT V1
                    (MAKE-SB-TERM 'FUNCALL
                                  (LIST V2
                                        (MAKE-SB-TERM 'FORM::ARGS
                                                      (DS-TEMP-REPT
                                                       (CK-REPT-REF V3))))))))
     ((LA-MATCH ((SBST::!ID! EPSILON)))
      (PROGN
        (LAM ((SBST::!ID! EPSILON)) (GOBBLE-TO-SLOT V1))
        (VALUE-TO-SLOT V0 0)))
     ((LA-MATCH ((SBST::!NUMBER!)))
      (PROGN
        (LAM ((SBST::!NUMBER! EPSILON)) (GOBBLE-TO-SLOT V1))
        (VALUE-TO-SLOT V0 1)))
     (T
      (INITIAL-ERROR
       '((SBST::!ID! SBST::|(| EPSILON)
        (SBST::< SBST::> SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
         SBST::!ID!)
        (SBST::!NUMBER! EPSILON)
        (SBST::IF SBST::!ID! SBST::!NUMBER! SBST::< SBST::IF SBST::CASE
         SBST::|(|)
        (SBST::CASE SBST::!ID! SBST::!NUMBER! SBST::< SBST::IF SBST::CASE
         SBST::|(|)
        (SBST::|(| SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
         SBST::!ID!)))))
    (VALUE-TO-SLOT V0 V1)
    (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
    (COND ((>
            (COND ((MEMBER (PEEK-FIRST) '(SBST::+ SBST::* SBST::@))
                   (SETQ MTEMP (PEEK-FIRST))
                   (CADR
                    (ASSOC (PEEK-FIRST)
                           '((SBST::JUX 40) (SBST::@ 30) (SBST::* 20)
                            (SBST::+ 10)))))
                  ((MEMBER (PEEK-FIRST)
                           '(SBST::|(| SBST::CASE SBST::IF SBST::<
                            SBST::!NUMBER! SBST::!ID!))
                   (SETQ MTEMP 'G27375)
                   40)
                  (T
                   0))
            RBP)
           (LOOP (CLEAN-VARIABLES (V1 V2 V3 V4 V5 V6 V7))
                 (CASE MTEMP
                   (SBST::@
                    (PROGN
                      (SLOT-TO-SLOT V2 V0)
                      (LAM ((SBST::@ EPSILON)) (GOBBLE-TOKEN))
                      (CODE-TO-SLOT V3 (!EXP! 31))
                      (VALUE-TO-SLOT V0 7)
                      (AS-TO-SLOT V1 (MAKE-SB-TERM 'FORM::ATSIGN (LIST V2 V3)))))
                   (SBST::*
                    (PROGN
                      (SLOT-TO-SLOT V2 V0)
                      (LAM ((SBST::* EPSILON)) (GOBBLE-TOKEN))
                      (CODE-TO-SLOT V3 (!EXP! 21))
                      (VALUE-TO-SLOT V0 6)
                      (AS-TO-SLOT V1 (MAKE-SB-TERM 'FORM::MULT (LIST V2 V3)))))
                   (SBST::+
                    (PROGN
                      (SLOT-TO-SLOT V2 V0)
                      (LAM ((SBST::+ EPSILON)) (GOBBLE-TOKEN))
                      (CODE-TO-SLOT V3 (!EXP! 11))
                      (VALUE-TO-SLOT V0 5)
                      (AS-TO-SLOT V1 (MAKE-SB-TERM 'FORM::PLUS (LIST V2 V3)))))
                   (G27375
                    (PROGN
                      (SLOT-TO-SLOT V2 V0)
                      (CODE-TO-SLOT V3 (!EXP! 41))
                      (VALUE-TO-SLOT V0 4)
                      (AS-TO-SLOT V1 (MAKE-SB-TERM 'FORM::APPL (LIST V2 V3)))))
                   (T
                    (MEDIAL-ERROR
                     '(SBST::|(| SBST::CASE SBST::IF SBST::< SBST::!NUMBER!
                      SBST::!ID! SBST::+ SBST::* SBST::@))))
                 (VALUE-TO-SLOT V0 V1) (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
                 (WHEN (<=
                   (COND ((MEMBER (PEEK-FIRST) '(SBST::+ SBST::* SBST::@))
                          (SETQ MTEMP (PEEK-FIRST))
                          (CADR
                           (ASSOC (PEEK-FIRST)
                                  '((SBST::JUX 40) (SBST::@ 30) (SBST::* 20)
                                   (SBST::+ 10)))))
                         ((MEMBER (PEEK-FIRST)
                                  '(SBST::|(| SBST::CASE SBST::IF SBST::<
                                   SBST::!NUMBER! SBST::!ID!))
                          (SETQ MTEMP 'G27375)
                          40)
                         (T
                          0))
                   RBP)
                   (RETURN NIL)))))
    V0)) 

