;;; -*- Mode: Lisp; Package: PASCAL -*-
(in-package "PASCAL")  ;; creates package for abstract syntax. 

(in-package "PASCAL")  ;; enters package for generated code.  

(use-package '("ERGOLISP" "OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( PASCAL-PARSE ))

(DEFPARAMETER PASCAL-ABS-SYN-PACKAGE
              (FIND-PACKAGE "PASCAL"))

(DEFUN PASCAL-PARSE
       (&KEY (NT 'PASCAL::PROGRAM)
             ERROR-THRESHOLD
             ASK-ABOUT-BAD-TOKENS
             (RETURN-ERRORS NIL)
             (STREAM NIL STREAMP)
             STRING
             FILE
             (EXHAUST-STREAM NIL))
       (COND (STREAM)
             (STRING (SETF STREAM
                           (MAKE-STRING-INPUT-STREAM STRING)))
             (FILE (SETF STREAM (OPEN FILE)))
             (T
              (ERROR
                "Must provide an input means -- either :stream, ~
			:string, or :file.")))
       (SETQ *PASCAL-KEYWORD-TABLE* (INIT-KEYWORD-TABLE PASCAL-KEYWORD-LIST))
       (LET ((*PARSER-ERROR* NIL)
             (*PARSER-ERROR-COUNT* 0)
             (*PARSER-RETURN-ERRORS* RETURN-ERRORS)
             (*PARSER-ABORT-THRESHOLD* ERROR-THRESHOLD)
             (*ASK-ABOUT-BAD-TOKENS* ASK-ABOUT-BAD-TOKENS)
             (*LEXICAL-STREAM* (OPEN-LEXICAL-STREAM (OPEN-PLACESTREAM STREAM)))
             (*HOLD-A1* NIL)
             (*HOLD-A2* NIL)
             (*HOLD-B1* NIL)
             (*HOLD-B2* NIL)
             (*ABS-SYN-PACKAGE* PASCAL-ABS-SYN-PACKAGE)
             (*READER-FUN* #'READER)
             (*APPLY-LEX-TERM-CONSTR-FUN* #'APPLY-LEXICAL-TERMINAL-CONSTRUCTOR)
             (*KEYWORD-TABLE* *PASCAL-KEYWORD-TABLE*)
             (*CLOSE-COMMENT-CHAR* PASCAL-CLOSE-COMMENT-CHAR)
             (*CASE-SENSITIVE* PASCAL-CASE-SENSITIVE))
         (INIT-LEXER *LEXICAL-STREAM*)
         (MULTIPLE-VALUE-BIND (ABS-SYNTAX ERROR-STRING
                                          ERROR-ARGS
                                          ERROR-PLACE)
                              (UNWIND-PROTECT
                                (CATCH 'PARSER-ABORT-CATCH
                                       (PROG1
                                         (CASE NT
                                               (PASCAL::PROGRAM (!PROGRAM! 0))
                                               (BLOCK (!BLOCK!
                                                       0))
                                               (PASCAL::FUNCTION-LIST
                                                 (!FUNCTION-LIST! 0))
                                               (PASCAL::PARAMETER-TYPE
                                                 (!PARAMETER-TYPE! 0))
                                               (PASCAL::HEADING (!HEADING! 0))
                                               (PASCAL::MY-TYPE (!MY-TYPE! 0))
                                               (PASCAL::FIELD-LIST
                                                 (!FIELD-LIST! 0))
                                               (PASCAL::SIMPLE-TYPE
                                                 (!SIMPLE-TYPE! 0))
                                               (PASCAL::STATEMENT
                                                 (!STATEMENT! 0))
                                               (PASCAL::REAL-STATEMENT
                                                 (!REAL-STATEMENT! 0))
                                               (EXP (!EXP! 0))
                                               (VARIABLE (!VARIABLE! 0))
                                               (PASCAL::CONSTANT
                                                 (!CONSTANT! 0))
                                               (T
                                                 (ERROR
                                                   "Unknown nonterminal ~A."
                                                   NT)))
                                         (WHEN
                                           (OR FILE STRING EXHAUST-STREAM)
                                           (CLET*
                                             (((TOKEN IGNORE PLACE)
                                                (PEEK-FIRST)))
                                             (UNLESS
                                               (OR (EQ TOKEN :EOF)
                                                   (EQ TOKEN 'EOF))
                                               (DO-SYNTAX-ERROR
                                                 "There is garbage at the end of your ~
			      file or string:~%~A"
                                                 PLACE))))))
                                (UNLESS STREAMP
                                        (CLOSE STREAM)))
                              (IF RETURN-ERRORS
                                  (VALUES ABS-SYNTAX
                                          *PARSER-ERROR*
                                          ERROR-PLACE
                                          ERROR-STRING
                                          ERROR-ARGS)
                                  (VALUES ABS-SYNTAX *PARSER-ERROR*)))))


(DEFUN !CONSTANT!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1)
         (ERGO-IGNORE-IF-UNUSED V0 V1)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 0)))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 1)))
           ((LA-MATCH ((SBST::!STRING!)))
             (PROGN (LAM ((SBST::!STRING! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 2)))
           ((LA-MATCH ((SBST::NULL)))
             (PROGN (LAM ((SBST::NULL EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'NULL (LIST)))))
           (T
             (INITIAL-ERROR
               '((SBST::NULL EPSILON) (SBST::!STRING! EPSILON)
                 (SBST::!ID! EPSILON) (SBST::!NUMBER! EPSILON)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !VARIABLE!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6
             V7
             V8)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6 V7 V8)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (OPT-PARSE
                      ((SBST::[ SBST::|(|
                                SBST::NOT
                                SBST::+
                                SBST::-
                                SBST::!NUMBER!
                                SBST::!ID!)
                        (SBST::|.| SBST::!ID!)
                        (SBST::^ EPSILON))
                      (PROGN
                        (ALT-PARSE
                          ((LA-MATCH ((SBST::[))) (VALUE-TO-SLOT V4 0)
                                                  (PROGN
                                                    (LAM ((SBST::[ EPSILON))
                                                         (GOBBLE-TOKEN))
                                                    (DOUBLEPLUS-PARSE
                                                      ((SBST::|(|
                                                         SBST::!NUMBER!
                                                         SBST::-
                                                         SBST::+
                                                         SBST::NOT
                                                         SBST::|(|
                                                         SBST::!ID!)
                                                        (SBST::NOT
                                                          SBST::!NUMBER!
                                                          SBST::-
                                                          SBST::+
                                                          SBST::NOT
                                                          SBST::|(|
                                                          SBST::!ID!)
                                                        (SBST::+ SBST::!NUMBER!
                                                                 SBST::-
                                                                 SBST::+
                                                                 SBST::NOT
                                                                 SBST::|(|
                                                                 SBST::!ID!)
                                                        (SBST::- SBST::!NUMBER!
                                                                 SBST::-
                                                                 SBST::+
                                                                 SBST::NOT
                                                                 SBST::|(|
                                                                 SBST::!ID!)
                                                        (SBST::!NUMBER! EPSILON
                                                                      SBST::DIV
                                                                      SBST::MOD
                                                                      SBST::*
                                                                      SBST::AND
                                                                      SBST::OR
                                                                      SBST::-
                                                                      SBST::+
                                                                      SBST::IN
                                                                      SBST::<>
                                                                      SBST::>=
                                                                      SBST::>
                                                                      SBST::=
                                                                      SBST::<
                                                                      SBST::<=)
                                                        (SBST::!ID! EPSILON
                                                                    SBST::^
                                                                    SBST::|.|
                                                                    SBST::[
                                                                    SBST::|(|
                                                                    SBST::DIV
                                                                    SBST::MOD
                                                                    SBST::*
                                                                    SBST::AND
                                                                    SBST::OR
                                                                    SBST::-
                                                                    SBST::+
                                                                    SBST::IN
                                                                    SBST::<>
                                                                    SBST::>=
                                                                    SBST::>
                                                                    SBST::=
                                                                    SBST::<
                                                                    SBST::<=))
                                                      (CODE-TO-SLOT V8
                                                                    (!EXP! 0))
                                                      V7
                                                      V8
                                                      'SBST::|,|)
                                                    (LAM ((SBST::] EPSILON))
                                                         (GOBBLE-TOKEN))
                                                    (AS-TO-SLOT V6
                                                                (MAKE-SB-TERM
                                                                  'PASCAL::ARRAY-REF
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V7))))))
                          ((LA-MATCH ((SBST::|.|))) (VALUE-TO-SLOT V4 1)
                                                    (PROGN
                                                      (LAM
                                                        ((SBST::|.| EPSILON))
                                                        (GOBBLE-TOKEN))
                                                      (LAM
                                                        ((SBST::!ID! EPSILON))
                                                        (GOBBLE-TO-SLOT V7))
                                                      (AS-TO-SLOT V6
                                                                  (MAKE-SB-TERM
                                                                    'PASCAL::REC-FIELD
                                                                    (LIST V7)))))
                          ((LA-MATCH ((SBST::^))) (VALUE-TO-SLOT V4 2)
                                                  (PROGN
                                                    (LAM ((SBST::^ EPSILON))
                                                         (GOBBLE-TOKEN))
                                                    (AS-TO-SLOT V6
                                                                (MAKE-SB-TERM
                                                                  'PASCAL::POINTER-REF
                                                                  (LIST)))))
                          (T
                            (INITIAL-ERROR
                              '((SBST::[ SBST::|(| SBST::NOT SBST::+ SBST::-
                                 SBST::!NUMBER! SBST::!ID!)
                                (SBST::|.| SBST::!ID!) (SBST::^ EPSILON)))))
                        (VALUE-TO-SLOT V5 V6))
                      V2)
                    (AS-TO-SLOT V3
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::VARIABLE-NULL-1
                                                  NIL)
                                    V5))
                    (AS-TO-SLOT V0
                                (IF (= V2 0)
                                    V1
                                    (MAKE-SB-TERM 'PASCAL::COMPLEX-REF
                                                  (LIST V1 V3))))))
           (T
             (INITIAL-ERROR
               '((SBST::!ID! EPSILON SBST::[ SBST::|.| SBST::^)))))
         NIL
         V0))


(DEFUN !EXP!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (LET (MTEMP
             V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (STACK-BRACKETS SBST::|(| EXP)
         (INITIALS
           ((LA-MATCH ((SBST::!ID! SBST::|(|)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::|(| EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE
                      ((SBST::|(| SBST::!NUMBER!
                                  SBST::-
                                  SBST::+
                                  SBST::NOT
                                  SBST::|(|
                                  SBST::!ID!)
                        (SBST::NOT SBST::!NUMBER!
                                   SBST::-
                                   SBST::+
                                   SBST::NOT
                                   SBST::|(|
                                   SBST::!ID!)
                        (SBST::+ SBST::!NUMBER!
                                 SBST::-
                                 SBST::+
                                 SBST::NOT
                                 SBST::|(|
                                 SBST::!ID!)
                        (SBST::- SBST::!NUMBER!
                                 SBST::-
                                 SBST::+
                                 SBST::NOT
                                 SBST::|(|
                                 SBST::!ID!)
                        (SBST::!NUMBER! EPSILON
                                        SBST::DIV
                                        SBST::MOD
                                        SBST::*
                                        SBST::AND
                                        SBST::OR
                                        SBST::-
                                        SBST::+
                                        SBST::IN
                                        SBST::<>
                                        SBST::>=
                                        SBST::>
                                        SBST::=
                                        SBST::<
                                        SBST::<=)
                        (SBST::!ID! EPSILON
                                    SBST::^
                                    SBST::|.|
                                    SBST::[
                                    SBST::|(|
                                    SBST::DIV
                                    SBST::MOD
                                    SBST::*
                                    SBST::AND
                                    SBST::OR
                                    SBST::-
                                    SBST::+
                                    SBST::IN
                                    SBST::<>
                                    SBST::>=
                                    SBST::>
                                    SBST::=
                                    SBST::<
                                    SBST::<=))
                      (CODE-TO-SLOT V4 (!EXP! 0))
                      V3
                      V4
                      'SBST::|,|)
                    (LAM ((SBST::|)| EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 19)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::FUNCTION-CALL
                                              (LIST V2
                                                    (MAKE-SB-TERM 'PASCAL::EXPS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V3))))))))
           ((LA-MATCH ((SBST::NOT)))
             (PROGN (LAM ((SBST::NOT EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 40))
                    (VALUE-TO-SLOT V0 18)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'NOT (LIST V2)))))
           ((LA-MATCH ((SBST::+)))
             (PROGN (LAM ((SBST::+ EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 40))
                    (VALUE-TO-SLOT V0 17)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::UNI-PLUS
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::-)))
             (PROGN (LAM ((SBST::- EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 40))
                    (VALUE-TO-SLOT V0 16)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::UNI-MINUS
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::!ID! EPSILON SBST::^ SBST::|.| SBST::[)))
             (PROGN (CODE-TO-SLOT V1 (!VARIABLE! 0))
                    (VALUE-TO-SLOT V0 0)))
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 1)))
           (T
             (INITIAL-ERROR
               '((SBST::!NUMBER! EPSILON)
                 (SBST::!ID! EPSILON SBST::^ SBST::|.| SBST::[ SBST::|(|)
                 (SBST::- SBST::|(| SBST::NOT SBST::+ SBST::- SBST::!NUMBER!
                  SBST::!ID!)
                 (SBST::+ SBST::|(| SBST::NOT SBST::+ SBST::- SBST::!NUMBER!
                  SBST::!ID!)
                 (SBST::NOT SBST::|(| SBST::NOT SBST::+ SBST::- SBST::!NUMBER!
                  SBST::!ID!)
                 (SBST::|(| SBST::|(| SBST::NOT SBST::+ SBST::- SBST::!NUMBER!
                  SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::<= SBST::< SBST::= SBST::> SBST::>=
                                   SBST::<> SBST::IN SBST::+ SBST::- SBST::OR
                                   SBST::AND SBST::* SBST::MOD SBST::DIV))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR
                           (ASSOC (PEEK-FIRST)
                                  '((SBST::AND 30) (SBST::MOD 30)
                                    (SBST::DIV 30) (SBST::/ 30) (SBST::* 30)
                                    (SBST::OR 20) (SBST::- 20) (SBST::+ 20)
                                    (SBST::IN 10) (SBST::<> 10) (SBST::>= 10)
                                    (SBST::> 10) (SBST::= 10) (SBST::<= 10)
                                    (SBST::< 10)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3 V4))
                      (CASE MTEMP
                            (SBST::DIV
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::DIV EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 31))
                                     (VALUE-TO-SLOT V0 15)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::DIV
                                                               (LIST V2 V3)))))
                            (SBST::MOD
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::MOD EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 31))
                                     (VALUE-TO-SLOT V0 14)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'MOD
                                                               (LIST V2 V3)))))
                            (SBST::*
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::* EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 31))
                                     (VALUE-TO-SLOT V0 13)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::TIMES
                                                               (LIST V2 V3)))))
                            (SBST::AND
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::AND EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 31))
                                     (VALUE-TO-SLOT V0 12)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'AND
                                                               (LIST V2 V3)))))
                            (SBST::OR
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::OR EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 21))
                                     (VALUE-TO-SLOT V0 11)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'OR
                                                               (LIST V2 V3)))))
                            (SBST::-
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::- EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 21))
                                     (VALUE-TO-SLOT V0 10)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::MINUS
                                                               (LIST V2 V3)))))
                            (SBST::+
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::+ EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 21))
                                     (VALUE-TO-SLOT V0 9)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::PLUS
                                                               (LIST V2 V3)))))
                            (SBST::IN
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::IN EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 8)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::IN
                                                               (LIST V2 V3)))))
                            (SBST::<>
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::<> EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 7)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::NE
                                                               (LIST V2 V3)))))
                            (SBST::>=
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::>= EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 6)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::GE
                                                               (LIST V2 V3)))))
                            (SBST::>
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::> EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 5)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::GT
                                                               (LIST V2 V3)))))
                            (SBST::=
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::= EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 4)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'EQ
                                                               (LIST V2 V3)))))
                            (SBST::<
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::< EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 3)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::LT
                                                               (LIST V2 V3)))))
                            (SBST::<=
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::<= EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 2)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'PASCAL::LE
                                                               (LIST V2 V3)))))
                            (T
                              (MEDIAL-ERROR
                                '(SBST::<= SBST::< SBST::= SBST::> SBST::>=
                                  SBST::<> SBST::IN SBST::+ SBST::- SBST::OR
                                  SBST::AND SBST::* SBST::MOD SBST::DIV))))
                      (VALUE-TO-SLOT V0 V1)
                      (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::<= SBST::< SBST::= SBST::>
                                           SBST::>= SBST::<> SBST::IN SBST::+
                                           SBST::- SBST::OR SBST::AND SBST::*
                                           SBST::MOD SBST::DIV))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR
                                   (ASSOC (PEEK-FIRST)
                                          '((SBST::AND 30) (SBST::MOD 30)
                                            (SBST::DIV 30) (SBST::/ 30)
                                            (SBST::* 30) (SBST::OR 20)
                                            (SBST::- 20) (SBST::+ 20)
                                            (SBST::IN 10) (SBST::<> 10)
                                            (SBST::>= 10) (SBST::> 10)
                                            (SBST::= 10) (SBST::<= 10)
                                            (SBST::< 10)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))


(DEFUN !REAL-STATEMENT!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6
             V7)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6 V7)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!ID! SBST::^
                                   SBST::|.|
                                   SBST::[
                                   SBST::|:=|)))
             (PROGN (CODE-TO-SLOT V2 (!VARIABLE! 0))
                    (LAM ((SBST::|:=| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXP! 0))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::ASSIGN
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::BEGIN)))
             (PROGN (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V3
                                    (!STATEMENT! 0))
                      V2
                      V3
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::STATS
                                              (DS-TEMP-REPT (CK-REPT-REF V2))))))
           ((LA-MATCH ((SBST::IF)))
             (PROGN (LAM ((SBST::IF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 0))
                    (LAM ((SBST::THEN EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!STATEMENT! 0))
                    (OPT-PARSE
                      ((SBST::ELSE SBST::GOTO
                                   SBST::WITH
                                   SBST::CASE
                                   SBST::FOR
                                   SBST::WHILE
                                   SBST::REPEAT
                                   SBST::IF
                                   SBST::BEGIN
                                   SBST::!ID!
                                   SBST::!NUMBER!))
                      (PROGN (LAM ((SBST::ELSE EPSILON))
                                  (GOBBLE-TOKEN))
                             (CODE-TO-SLOT V5 (!STATEMENT! 0)))
                      V4)
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (IF (= V4 0)
                                    (MAKE-SB-TERM 'PASCAL::IF-THEN
                                                  (LIST V2 V3))
                                    (MAKE-SB-TERM 'PASCAL::IF-THEN-ELSE
                                                  (LIST V2 V3 V5))))))
           ((LA-MATCH ((SBST::REPEAT)))
             (PROGN (LAM ((SBST::REPEAT EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V3 (!STATEMENT! 0))
                      V2
                      V3
                      'SBST::|;|)
                    (LAM ((SBST::UNTIL EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V4 (!EXP! 0))
                    (VALUE-TO-SLOT V0 4)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::REPEAT
                                              (LIST
                                                (MAKE-SB-TERM 'PASCAL::STATS
                                                              (DS-TEMP-REPT
                                                                (CK-REPT-REF V2)))
                                                V4)))))
           ((LA-MATCH ((SBST::WHILE)))
             (PROGN (LAM ((SBST::WHILE EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 0))
                    (LAM ((SBST::DO EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!STATEMENT! 0))
                    (VALUE-TO-SLOT V0 5)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::WHILE
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::FOR)))
             (PROGN (LAM ((SBST::FOR EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::|:=| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXP! 0))
                    (ALT-PARSE
                      ((LA-MATCH ((SBST::TO))) (VALUE-TO-SLOT V4 0)
                                               (LAM ((SBST::TO EPSILON))
                                                    (GOBBLE-TOKEN)))
                      ((LA-MATCH ((SBST::DOWNTO))) (VALUE-TO-SLOT V4 1)
                                                   (LAM
                                                     ((SBST::DOWNTO EPSILON))
                                                     (GOBBLE-TOKEN)))
                      (T
                        (INITIAL-ERROR
                          '((SBST::TO EPSILON) (SBST::DOWNTO EPSILON)))))
                    (CODE-TO-SLOT V5 (!EXP! 0))
                    (LAM ((SBST::DO EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V6 (!STATEMENT! 0))
                    (VALUE-TO-SLOT V0 6)
                    (AS-TO-SLOT V1
                                (CASE V4
                                      (1
                                        (MAKE-SB-TERM 'PASCAL::FOR-DOWNTO
                                                      (LIST V2 V3 V5 V6)))
                                      (0
                                        (MAKE-SB-TERM 'PASCAL::FOR-TO
                                                      (LIST V2 V3 V5 V6)))))))
           ((LA-MATCH ((SBST::CASE)))
             (PROGN (LAM ((SBST::CASE EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 0))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::NULL SBST::|,|
                                   SBST::GOTO
                                   SBST::WITH
                                   SBST::CASE
                                   SBST::FOR
                                   SBST::WHILE
                                   SBST::REPEAT
                                   SBST::IF
                                   SBST::BEGIN
                                   SBST::!ID!
                                   SBST::!NUMBER!)
                        (SBST::!STRING! SBST::|,|
                                        SBST::GOTO
                                        SBST::WITH
                                        SBST::CASE
                                        SBST::FOR
                                        SBST::WHILE
                                        SBST::REPEAT
                                        SBST::IF
                                        SBST::BEGIN
                                        SBST::!ID!
                                        SBST::!NUMBER!)
                        (SBST::!ID! SBST::|,|
                                    SBST::GOTO
                                    SBST::WITH
                                    SBST::CASE
                                    SBST::FOR
                                    SBST::WHILE
                                    SBST::REPEAT
                                    SBST::IF
                                    SBST::BEGIN
                                    SBST::!ID!
                                    SBST::!NUMBER!)
                        (SBST::!NUMBER! SBST::|,|
                                        SBST::GOTO
                                        SBST::WITH
                                        SBST::CASE
                                        SBST::FOR
                                        SBST::WHILE
                                        SBST::REPEAT
                                        SBST::IF
                                        SBST::BEGIN
                                        SBST::!ID!
                                        SBST::!NUMBER!))
                      (PROGN
                        (DOUBLEPLUS-PARSE
                          ((SBST::NULL EPSILON) (SBST::!STRING! EPSILON)
                                                (SBST::!ID! EPSILON)
                                                (SBST::!NUMBER! EPSILON))
                          (CODE-TO-SLOT V6
                                        (!CONSTANT! 0))
                          V5
                          V6
                          'SBST::|,|)
                        (CODE-TO-SLOT V7 (!STATEMENT! 0))
                        (AS-TO-SLOT V4
                                    (MAKE-SB-TERM 'PASCAL::CASE-ENTRY
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::CONDS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V5)))
                                                    V7))))
                      V3
                      V4
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 7)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CASE
                                              (LIST V2
                                                    (MAKE-SB-TERM
                                                      'PASCAL::CHOICES
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V3))))))))
           ((LA-MATCH ((SBST::WITH)))
             (PROGN (LAM ((SBST::WITH EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE
                      ((SBST::!ID! EPSILON SBST::^ SBST::|.| SBST::[))
                      (CODE-TO-SLOT V3 (!VARIABLE! 0))
                      V2
                      V3
                      'SBST::|,|)
                    (LAM ((SBST::DO EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V4 (!STATEMENT! 0))
                    (VALUE-TO-SLOT V0 8)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::WITH
                                              (LIST
                                                (MAKE-SB-TERM 'PASCAL::VARS
                                                              (DS-TEMP-REPT
                                                                (CK-REPT-REF V2)))
                                                V4)))))
           ((LA-MATCH ((SBST::GOTO)))
             (PROGN (LAM ((SBST::GOTO EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (VALUE-TO-SLOT V0 9)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::GOTO
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::!ID! EPSILON SBST::|(|)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE
                      ((SBST::|(| SBST::|(|
                                  SBST::NOT
                                  SBST::+
                                  SBST::-
                                  SBST::!NUMBER!
                                  SBST::!ID!))
                      (PROGN (LAM ((SBST::|(| EPSILON))
                                  (GOBBLE-TOKEN))
                             (DOUBLEPLUS-PARSE
                               ((SBST::|(| SBST::!NUMBER!
                                           SBST::-
                                           SBST::+
                                           SBST::NOT
                                           SBST::|(|
                                           SBST::!ID!)
                                 (SBST::NOT SBST::!NUMBER!
                                            SBST::-
                                            SBST::+
                                            SBST::NOT
                                            SBST::|(|
                                            SBST::!ID!)
                                 (SBST::+ SBST::!NUMBER!
                                          SBST::-
                                          SBST::+
                                          SBST::NOT
                                          SBST::|(|
                                          SBST::!ID!)
                                 (SBST::- SBST::!NUMBER!
                                          SBST::-
                                          SBST::+
                                          SBST::NOT
                                          SBST::|(|
                                          SBST::!ID!)
                                 (SBST::!NUMBER! EPSILON
                                                 SBST::DIV
                                                 SBST::MOD
                                                 SBST::*
                                                 SBST::AND
                                                 SBST::OR
                                                 SBST::-
                                                 SBST::+
                                                 SBST::IN
                                                 SBST::<>
                                                 SBST::>=
                                                 SBST::>
                                                 SBST::=
                                                 SBST::<
                                                 SBST::<=)
                                 (SBST::!ID! EPSILON
                                             SBST::^
                                             SBST::|.|
                                             SBST::[
                                             SBST::|(|
                                             SBST::DIV
                                             SBST::MOD
                                             SBST::*
                                             SBST::AND
                                             SBST::OR
                                             SBST::-
                                             SBST::+
                                             SBST::IN
                                             SBST::<>
                                             SBST::>=
                                             SBST::>
                                             SBST::=
                                             SBST::<
                                             SBST::<=))
                               (CODE-TO-SLOT V5 (!EXP! 0))
                               V4
                               V5
                               'SBST::|,|)
                             (LAM ((SBST::|)| EPSILON))
                                  (GOBBLE-TOKEN)))
                      V3)
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::PROC-CALL
                                              (LIST V2
                                                    (IF (= V3 0)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::EXPS
                                                          NIL)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::EXPS
                                                          (DS-TEMP-REPT
                                                            (CK-REPT-REF V4)))))))))
           (T
             (INITIAL-ERROR
               '((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                 (SBST::CASE SBST::!ID! SBST::!NUMBER! SBST::- SBST::+
                  SBST::NOT SBST::|(|)
                 (SBST::FOR SBST::!ID!)
                 (SBST::WHILE SBST::!ID! SBST::!NUMBER! SBST::- SBST::+
                  SBST::NOT SBST::|(|)
                 (SBST::REPEAT SBST::UNTIL SBST::GOTO SBST::WITH SBST::CASE
                  SBST::FOR SBST::WHILE SBST::REPEAT SBST::IF SBST::BEGIN
                  SBST::!ID! SBST::!NUMBER!)
                 (SBST::IF SBST::!ID! SBST::!NUMBER! SBST::- SBST::+ SBST::NOT
                  SBST::|(|)
                 (SBST::!ID! EPSILON SBST::|(| SBST::^ SBST::|.| SBST::[
                  SBST::|:=|)
                 (SBST::BEGIN SBST::END SBST::GOTO SBST::WITH SBST::CASE
                  SBST::FOR SBST::WHILE SBST::REPEAT SBST::IF SBST::BEGIN
                  SBST::!ID! SBST::!NUMBER!)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !STATEMENT!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V1 1)
                    (CODE-TO-SLOT V3
                                  (!REAL-STATEMENT! 0))
                    (AS-TO-SLOT V0
                                (IF (= V1 0)
                                    V3
                                    (MAKE-SB-TERM 'PASCAL::NUMBER-STAT
                                                  (LIST V2 V3))))))
           ((LA-MATCH
              ((SBST::!ID!) (SBST::BEGIN)
                            (SBST::IF)
                            (SBST::REPEAT)
                            (SBST::WHILE)
                            (SBST::FOR)
                            (SBST::CASE)
                            (SBST::WITH)
                            (SBST::GOTO)))
             (PROGN (CODE-TO-SLOT V3 (!REAL-STATEMENT! 0))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V0
                                (IF (= V1 0)
                                    V3
                                    (MAKE-SB-TERM 'PASCAL::NUMBER-STAT
                                                  (LIST V2 V3))))))
           (T
             (INITIAL-ERROR
               '((SBST::!NUMBER! SBST::|:|) (SBST::GOTO SBST::!NUMBER!)
                 (SBST::WITH SBST::!ID!)
                 (SBST::CASE SBST::!NUMBER! SBST::- SBST::+ SBST::NOT SBST::|(|
                  SBST::!ID!)
                 (SBST::FOR SBST::!ID!)
                 (SBST::WHILE SBST::!NUMBER! SBST::- SBST::+ SBST::NOT
                  SBST::|(| SBST::!ID!)
                 (SBST::REPEAT SBST::UNTIL SBST::!NUMBER! SBST::BEGIN SBST::IF
                  SBST::REPEAT SBST::WHILE SBST::FOR SBST::CASE SBST::WITH
                  SBST::GOTO SBST::!ID!)
                 (SBST::IF SBST::!NUMBER! SBST::- SBST::+ SBST::NOT SBST::|(|
                  SBST::!ID!)
                 (SBST::BEGIN SBST::END SBST::!NUMBER! SBST::BEGIN SBST::IF
                  SBST::REPEAT SBST::WHILE SBST::FOR SBST::CASE SBST::WITH
                  SBST::GOTO SBST::!ID!)
                 (SBST::!ID! EPSILON SBST::|(| SBST::[ SBST::|.| SBST::^
                  SBST::|:=|)))))
         NIL
         V0))


(DEFUN !SIMPLE-TYPE!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::|(|)))
             (PROGN (LAM ((SBST::|(| EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                      (LAM ((SBST::!ID! EPSILON))
                                           (GOBBLE-TO-SLOT V3))
                                      V2
                                      V3
                                      'SBST::|,|)
                    (LAM ((SBST::|)| EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::ENUMERATED-TYPE
                                              (DS-TEMP-REPT (CK-REPT-REF V2))))))
           ((LA-MATCH
              ((SBST::NULL) (SBST::!STRING!)
                            (SBST::!ID! SBST::|..|)
                            (SBST::!NUMBER!)))
             (PROGN (CODE-TO-SLOT V2 (!CONSTANT! 0))
                    (LAM ((SBST::|..| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!CONSTANT! 0))
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::RANGE
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::!ID! EPSILON)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 0)))
           (T
             (INITIAL-ERROR
               '((SBST::NULL SBST::|..|) (SBST::!STRING! SBST::|..|)
                 (SBST::!ID! SBST::|..| EPSILON) (SBST::!NUMBER! SBST::|..|)
                 (SBST::|(| SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !FIELD-LIST!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6
             V7
             V8
             V9
             V10
             V11
             V12
             V13
             V14
             V15
             V16
             V17
             V18
             V19
             V20
             V21)
         (ERGO-IGNORE-IF-UNUSED V0
                                V1
                                V2
                                V3
                                V4
                                V5
                                V6
                                V7
                                V8
                                V9
                                V10
                                V11
                                V12
                                V13
                                V14
                                V15
                                V16
                                V17
                                V18
                                V19
                                V20
                                V21)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V5))
                    (CONS-TO-SLOT V4 V5)
                    (GEN-STAR-PARSE ((SBST::|,| SBST::!ID!))
                                    (PROGN
                                      (LAM ((SBST::|,| EPSILON))
                                           (GOBBLE-TOKEN))
                                      (LAM ((SBST::!ID! EPSILON))
                                           (GOBBLE-TO-SLOT V5)))
                                    V4
                                    V5)
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V6 (!MY-TYPE! 0))
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (AS-TO-SLOT V3
                                (MAKE-SB-TERM 'PASCAL::FIELDS
                                              (LIST
                                                (MAKE-SB-TERM 'PASCAL::IDS
                                                              (DS-TEMP-REPT
                                                                (CK-REPT-REF V4)))
                                                V6)))
                    (CONS-TO-SLOT V2 V3)
                    (GEN-STAR-PARSE ((SBST::!ID! SBST::|,| SBST::|:|))
                                    (PROGN
                                      (DOUBLEPLUS-PARSE
                                        ((SBST::!ID! EPSILON))
                                        (LAM ((SBST::!ID! EPSILON))
                                             (GOBBLE-TO-SLOT V5))
                                        V4
                                        V5
                                        'SBST::|,|)
                                      (LAM ((SBST::|:| EPSILON))
                                           (GOBBLE-TOKEN))
                                      (CODE-TO-SLOT V6
                                                    (!MY-TYPE! 0))
                                      (LAM ((SBST::|;| EPSILON))
                                           (GOBBLE-TOKEN))
                                      (AS-TO-SLOT V3
                                                  (MAKE-SB-TERM 'PASCAL::FIELDS
                                                                (LIST
                                                                  (MAKE-SB-TERM
                                                                    'PASCAL::IDS
                                                                    (DS-TEMP-REPT
                                                                      (CK-REPT-REF
                                                                      V4)))
                                                                  V6))))
                                    V2
                                    V3)
                    (OPT-PARSE ((SBST::CASE SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::CASE EPSILON))
                                      (GOBBLE-TOKEN))
                                 (OPT-PARSE ((SBST::!ID! SBST::|:|))
                                            (PROGN
                                              (LAM ((SBST::!ID! EPSILON))
                                                   (GOBBLE-TO-SLOT V13))
                                              (LAM ((SBST::|:| EPSILON))
                                                   (GOBBLE-TOKEN))
                                              (VALUE-TO-SLOT V12 V13))
                                            V10)
                                 (AS-TO-SLOT V11
                                             (IF (= V10 0)
                                                 (MAKE-SB-TERM
                                                   'PASCAL::FIELD-LIST-NULL-2
                                                   NIL)
                                                 V12))
                                 (LAM ((SBST::!ID! EPSILON))
                                      (GOBBLE-TO-SLOT V14))
                                 (LAM ((SBST::OF EPSILON))
                                      (GOBBLE-TOKEN))
                                 (DOUBLEPLUS-PARSE
                                   ((SBST::NULL SBST::|,| SBST::|:|)
                                     (SBST::!STRING! SBST::|,| SBST::|:|)
                                     (SBST::!ID! SBST::|,| SBST::|:|)
                                     (SBST::!NUMBER! SBST::|,| SBST::|:|))
                                   (PROGN
                                     (DOUBLEPLUS-PARSE
                                       ((SBST::NULL EPSILON)
                                         (SBST::!STRING! EPSILON)
                                         (SBST::!ID! EPSILON)
                                         (SBST::!NUMBER! EPSILON))
                                       (CODE-TO-SLOT V18
                                                     (!CONSTANT! 0))
                                       V17
                                       V18
                                       'SBST::|,|)
                                     (LAM ((SBST::|:| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (LAM ((SBST::|(| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (OPT-PARSE
                                       ((SBST::CASE SBST::!ID!)
                                         (SBST::!ID! SBST::|:| SBST::|,|))
                                       (CODE-TO-SLOT V21
                                                     (!FIELD-LIST! 0))
                                       V19)
                                     (AS-TO-SLOT V20
                                                 (IF (= V19 0)
                                                     (MAKE-SB-TERM
                                                       'PASCAL::FIELD-LIST-NULL-3
                                                       NIL)
                                                     V21))
                                     (LAM ((SBST::|)| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (AS-TO-SLOT V16
                                                 (MAKE-SB-TERM 'PASCAL::BRANCH
                                                               (LIST
                                                                 (MAKE-SB-TERM
                                                                   'PASCAL::CONSTANTS
                                                                   (DS-TEMP-REPT
                                                                     (CK-REPT-REF
                                                                      V17)))
                                                                 V20))))
                                   V15
                                   V16
                                   'SBST::|;|)
                                 (AS-TO-SLOT V9
                                             (MAKE-SB-TERM
                                               'PASCAL::VARIANT-PART
                                               (LIST V11
                                                     V14
                                                     (MAKE-SB-TERM
                                                       'PASCAL::BRANCHES
                                                       (DS-TEMP-REPT
                                                         (CK-REPT-REF V15)))))))
                               V7)
                    (AS-TO-SLOT V8
                                (IF (= V7 0)
                                    (MAKE-SB-TERM 'PASCAL::FIELD-LIST-NULL-1
                                                  NIL)
                                    V9))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::FIELD-LIST
                                              (LIST
                                                (MAKE-SB-TERM 'PASCAL::FIELDS
                                                              (DS-TEMP-REPT
                                                                (CK-REPT-REF V2)))
                                                V8)))))
           ((LA-MATCH ((SBST::CASE)))
             (PROGN (LAM ((SBST::CASE EPSILON))
                         (GOBBLE-TOKEN))
                    (OPT-PARSE ((SBST::!ID! SBST::|:|))
                               (PROGN
                                 (LAM ((SBST::!ID! EPSILON))
                                      (GOBBLE-TO-SLOT V5))
                                 (LAM ((SBST::|:| EPSILON))
                                      (GOBBLE-TOKEN))
                                 (VALUE-TO-SLOT V4 V5))
                               V2)
                    (AS-TO-SLOT V3
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::FIELD-LIST-NULL-4
                                                  NIL)
                                    V4))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V6))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE
                      ((SBST::NULL SBST::|,| SBST::|:|)
                        (SBST::!STRING! SBST::|,| SBST::|:|)
                        (SBST::!ID! SBST::|,| SBST::|:|)
                        (SBST::!NUMBER! SBST::|,| SBST::|:|))
                      (PROGN
                        (DOUBLEPLUS-PARSE
                          ((SBST::NULL EPSILON) (SBST::!STRING! EPSILON)
                                                (SBST::!ID! EPSILON)
                                                (SBST::!NUMBER! EPSILON))
                          (CODE-TO-SLOT V10 (!CONSTANT! 0))
                          V9
                          V10
                          'SBST::|,|)
                        (LAM ((SBST::|:| EPSILON))
                             (GOBBLE-TOKEN))
                        (LAM ((SBST::|(| EPSILON))
                             (GOBBLE-TOKEN))
                        (OPT-PARSE
                          ((SBST::CASE SBST::!ID!)
                            (SBST::!ID! SBST::|:| SBST::|,|))
                          (CODE-TO-SLOT V13 (!FIELD-LIST! 0))
                          V11)
                        (AS-TO-SLOT V12
                                    (IF (= V11 0)
                                        (MAKE-SB-TERM
                                          'PASCAL::FIELD-LIST-NULL-5
                                          NIL)
                                        V13))
                        (LAM ((SBST::|)| EPSILON))
                             (GOBBLE-TOKEN))
                        (AS-TO-SLOT V8
                                    (MAKE-SB-TERM 'PASCAL::BRANCH
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::CONSTANTS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V9)))
                                                    V12))))
                      V7
                      V8
                      'SBST::|;|)
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::VARIANT-PART
                                              (LIST V3
                                                    V6
                                                    (MAKE-SB-TERM
                                                      'PASCAL::BRANCHES
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V7))))))))
           (T
             (INITIAL-ERROR
               '((SBST::CASE SBST::!ID!) (SBST::!ID! SBST::|:| SBST::|,|)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !MY-TYPE!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::^)))
             (PROGN (LAM ((SBST::^ EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::POINTER
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::SET)))
             (PROGN (LAM ((SBST::SET EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3
                                  (!SIMPLE-TYPE! 0))
                    (VALUE-TO-SLOT V2 0)
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'SET (LIST V3))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-SET
                                                  (LIST V3))))))
           ((LA-MATCH ((SBST::PACKED SBST::SET)))
             (PROGN (LAM ((SBST::PACKED EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 1)
                    (LAM ((SBST::SET EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!SIMPLE-TYPE! 0))
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'SET (LIST V3))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-SET
                                                  (LIST V3))))))
           ((LA-MATCH ((SBST::ARRAY)))
             (PROGN (LAM ((SBST::ARRAY EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::[ EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE
                      ((SBST::NULL SBST::|..|) (SBST::!STRING! SBST::|..|)
                                               (SBST::!NUMBER! SBST::|..|)
                                               (SBST::|(| SBST::!ID!)
                                               (SBST::!ID! SBST::|..| EPSILON))
                      (CODE-TO-SLOT V4 (!SIMPLE-TYPE! 0))
                      V3
                      V4
                      'SBST::|,|)
                    (LAM ((SBST::] EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V5 (!MY-TYPE! 0))
                    (VALUE-TO-SLOT V2 0)
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'ARRAY
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::SUBSCRIPTS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V3)))
                                                    V5))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-ARRAY
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::SUBSCRIPTS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V3)))
                                                    V5))))))
           ((LA-MATCH ((SBST::PACKED SBST::ARRAY)))
             (PROGN (LAM ((SBST::PACKED EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 1)
                    (LAM ((SBST::ARRAY EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::[ EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE
                      ((SBST::NULL SBST::|..|) (SBST::!STRING! SBST::|..|)
                                               (SBST::!NUMBER! SBST::|..|)
                                               (SBST::|(| SBST::!ID!)
                                               (SBST::!ID! SBST::|..| EPSILON))
                      (CODE-TO-SLOT V4 (!SIMPLE-TYPE! 0))
                      V3
                      V4
                      'SBST::|,|)
                    (LAM ((SBST::] EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V5 (!MY-TYPE! 0))
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'ARRAY
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::SUBSCRIPTS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V3)))
                                                    V5))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-ARRAY
                                                  (LIST
                                                    (MAKE-SB-TERM
                                                      'PASCAL::SUBSCRIPTS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V3)))
                                                    V5))))))
           ((LA-MATCH ((SBST::RECORD)))
             (PROGN (LAM ((SBST::RECORD EPSILON))
                         (GOBBLE-TOKEN))
                    (OPT-PARSE
                      ((SBST::CASE SBST::!ID!)
                        (SBST::!ID! SBST::|:| SBST::|,|))
                      (PROGN (CODE-TO-SLOT V5
                                           (!FIELD-LIST! 0))
                             (AS-TO-SLOT V5
                                         (MAKE-SB-TERM 'PASCAL::FLIST
                                                       (LIST V5))))
                      V3)
                    (AS-TO-SLOT V4
                                (IF (= V3 0)
                                    (MAKE-SB-TERM 'PASCAL::MY-TYPE-NULL-1
                                                  NIL)
                                    V5))
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 0)
                    (VALUE-TO-SLOT V0 4)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::RECORD
                                                  (LIST V4))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-RECORD
                                                  (LIST V4))))))
           ((LA-MATCH ((SBST::PACKED SBST::RECORD)))
             (PROGN (LAM ((SBST::PACKED EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 1)
                    (LAM ((SBST::RECORD EPSILON))
                         (GOBBLE-TOKEN))
                    (OPT-PARSE
                      ((SBST::CASE SBST::!ID!)
                        (SBST::!ID! SBST::|:| SBST::|,|))
                      (PROGN (CODE-TO-SLOT V5
                                           (!FIELD-LIST! 0))
                             (AS-TO-SLOT V5
                                         (MAKE-SB-TERM 'PASCAL::FLIST
                                                       (LIST V5))))
                      V3)
                    (AS-TO-SLOT V4
                                (IF (= V3 0)
                                    (MAKE-SB-TERM 'PASCAL::MY-TYPE-NULL-1
                                                  NIL)
                                    V5))
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 4)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::RECORD
                                                  (LIST V4))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-RECORD
                                                  (LIST V4))))))
           ((LA-MATCH ((SBST::FILE)))
             (PROGN (LAM ((SBST::FILE EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!MY-TYPE! 0))
                    (VALUE-TO-SLOT V2 0)
                    (VALUE-TO-SLOT V0 5)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::FILE
                                                  (LIST V3))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-FILE
                                                  (LIST V3))))))
           ((LA-MATCH ((SBST::PACKED SBST::FILE)))
             (PROGN (LAM ((SBST::PACKED EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 1)
                    (LAM ((SBST::FILE EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::OF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!MY-TYPE! 0))
                    (VALUE-TO-SLOT V0 5)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::FILE
                                                  (LIST V3))
                                    (MAKE-SB-TERM 'PASCAL::PACKED-FILE
                                                  (LIST V3))))))
           ((LA-MATCH
              ((SBST::!ID!) (SBST::|(|)
                            (SBST::!NUMBER!)
                            (SBST::!STRING!)
                            (SBST::NULL)))
             (PROGN (CODE-TO-SLOT V1 (!SIMPLE-TYPE! 0))
                    (VALUE-TO-SLOT V0 0)))
           (T
             (INITIAL-ERROR
               '((SBST::PACKED SBST::FILE SBST::RECORD SBST::ARRAY SBST::SET)
                 (SBST::FILE SBST::OF)
                 (SBST::RECORD SBST::END SBST::!ID! SBST::CASE)
                 (SBST::ARRAY SBST::[) (SBST::SET SBST::OF)
                 (SBST::^ SBST::!ID!) (SBST::NULL SBST::|..|)
                 (SBST::!STRING! SBST::|..|) (SBST::!NUMBER! SBST::|..|)
                 (SBST::|(| SBST::!ID!) (SBST::!ID! SBST::|..| EPSILON)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !PARAMETER-TYPE!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::VAR)))
             (PROGN (LAM ((SBST::VAR EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V2 1)
                    (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                      (LAM ((SBST::!ID! EPSILON))
                                           (GOBBLE-TO-SLOT V4))
                                      V3
                                      V4
                                      'SBST::|,|)
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V5))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::VAL
                                                  (LIST
                                                    (MAKE-SB-TERM 'PASCAL::IDS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V3)))
                                                    V5))
                                    (MAKE-SB-TERM 'REF
                                                  (LIST
                                                    (MAKE-SB-TERM 'PASCAL::IDS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V3)))
                                                    V5))))))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V4))
                    (CONS-TO-SLOT V3 V4)
                    (GEN-STAR-PARSE ((SBST::|,| SBST::!ID!))
                                    (PROGN
                                      (LAM ((SBST::|,| EPSILON))
                                           (GOBBLE-TOKEN))
                                      (LAM ((SBST::!ID! EPSILON))
                                           (GOBBLE-TO-SLOT V4)))
                                    V3
                                    V4)
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V5))
                    (VALUE-TO-SLOT V2 0)
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'PASCAL::VAL
                                                  (LIST
                                                    (MAKE-SB-TERM 'PASCAL::IDS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V3)))
                                                    V5))
                                    (MAKE-SB-TERM 'REF
                                                  (LIST
                                                    (MAKE-SB-TERM 'PASCAL::IDS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V3)))
                                                    V5))))))
           ((LA-MATCH ((SBST::PROCEDURE) (SBST::FUNCTION)))
             (PROGN (CODE-TO-SLOT V1 (!HEADING! 0))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::PROC-PARAM
                                              (LIST V1)))))
           (T
             (INITIAL-ERROR
               '((SBST::PROCEDURE SBST::!ID!) (SBST::FUNCTION SBST::!ID!)
                 (SBST::!ID! SBST::|:| SBST::|,|) (SBST::VAR SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !HEADING!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::PROCEDURE)))
             (PROGN (LAM ((SBST::PROCEDURE EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE
                      ((SBST::|(| SBST::FUNCTION
                                  SBST::PROCEDURE
                                  SBST::VAR
                                  SBST::!ID!))
                      (PROGN (LAM ((SBST::|(| EPSILON))
                                  (GOBBLE-TOKEN))
                             (DOUBLEPLUS-PARSE
                               ((SBST::FUNCTION SBST::!ID!)
                                 (SBST::PROCEDURE SBST::!ID!)
                                 (SBST::VAR SBST::!ID!)
                                 (SBST::!ID! SBST::|:|
                                             SBST::|,|))
                               (CODE-TO-SLOT V5
                                             (!PARAMETER-TYPE! 0))
                               V4
                               V5
                               'SBST::|;|)
                             (LAM ((SBST::|)| EPSILON))
                                  (GOBBLE-TOKEN)))
                      V3)
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::PROC-PARAM
                                              (LIST V2
                                                    (IF (= V3 0)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          NIL)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          (DS-TEMP-REPT
                                                            (CK-REPT-REF V4)))))))))
           ((LA-MATCH ((SBST::FUNCTION)))
             (PROGN (LAM ((SBST::FUNCTION EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE
                      ((SBST::|(| SBST::FUNCTION
                                  SBST::PROCEDURE
                                  SBST::VAR
                                  SBST::!ID!))
                      (PROGN (LAM ((SBST::|(| EPSILON))
                                  (GOBBLE-TOKEN))
                             (DOUBLEPLUS-PARSE
                               ((SBST::FUNCTION SBST::!ID!)
                                 (SBST::PROCEDURE SBST::!ID!)
                                 (SBST::VAR SBST::!ID!)
                                 (SBST::!ID! SBST::|:|
                                             SBST::|,|))
                               (CODE-TO-SLOT V5
                                             (!PARAMETER-TYPE! 0))
                               V4
                               V5
                               'SBST::|;|)
                             (LAM ((SBST::|)| EPSILON))
                                  (GOBBLE-TOKEN)))
                      V3)
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V6))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::FUNCT-PARAM
                                              (LIST V2
                                                    (IF (= V3 0)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          NIL)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          (DS-TEMP-REPT
                                                            (CK-REPT-REF V4))))
                                                    V6)))))
           (T
             (INITIAL-ERROR
               '((SBST::FUNCTION SBST::!ID!) (SBST::PROCEDURE SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !FUNCTION-LIST!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6
             V7)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6 V7)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::PROCEDURE)))
             (PROGN (LAM ((SBST::PROCEDURE EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE
                      ((SBST::|(| SBST::FUNCTION
                                  SBST::PROCEDURE
                                  SBST::VAR
                                  SBST::!ID!))
                      (PROGN (LAM ((SBST::|(| EPSILON))
                                  (GOBBLE-TOKEN))
                             (DOUBLEPLUS-PARSE
                               ((SBST::FUNCTION SBST::!ID!)
                                 (SBST::PROCEDURE SBST::!ID!)
                                 (SBST::VAR SBST::!ID!)
                                 (SBST::!ID! SBST::|:|
                                             SBST::|,|))
                               (CODE-TO-SLOT V5
                                             (!PARAMETER-TYPE! 0))
                               V4
                               V5
                               'SBST::|;|)
                             (LAM ((SBST::|)| EPSILON))
                                  (GOBBLE-TOKEN)))
                      V3)
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V6 (!BLOCK! 0))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::PROC-DEF
                                              (LIST V2
                                                    (IF (= V3 0)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          NIL)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          (DS-TEMP-REPT
                                                            (CK-REPT-REF V4))))
                                                    V6)))))
           ((LA-MATCH ((SBST::FUNCTION)))
             (PROGN (LAM ((SBST::FUNCTION EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE
                      ((SBST::|(| SBST::FUNCTION
                                  SBST::PROCEDURE
                                  SBST::VAR
                                  SBST::!ID!))
                      (PROGN (LAM ((SBST::|(| EPSILON))
                                  (GOBBLE-TOKEN))
                             (DOUBLEPLUS-PARSE
                               ((SBST::FUNCTION SBST::!ID!)
                                 (SBST::PROCEDURE SBST::!ID!)
                                 (SBST::VAR SBST::!ID!)
                                 (SBST::!ID! SBST::|:|
                                             SBST::|,|))
                               (CODE-TO-SLOT V5
                                             (!PARAMETER-TYPE! 0))
                               V4
                               V5
                               'SBST::|;|)
                             (LAM ((SBST::|)| EPSILON))
                                  (GOBBLE-TOKEN)))
                      V3)
                    (LAM ((SBST::|:| EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V6))
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V7 (!BLOCK! 0))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'PASCAL::FUNCT-DEF
                                              (LIST V2
                                                    (IF (= V3 0)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          NIL)
                                                        (MAKE-SB-TERM
                                                          'PASCAL::PARAMS
                                                          (DS-TEMP-REPT
                                                            (CK-REPT-REF V4))))
                                                    V6
                                                    V7)))))
           (T
             (INITIAL-ERROR
               '((SBST::FUNCTION SBST::!ID!) (SBST::PROCEDURE SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         V0))


(DEFUN !BLOCK!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5
             V6
             V7
             V8
             V9
             V10
             V11
             V12
             V13
             V14
             V15
             V16
             V17
             V18
             V19
             V20
             V21
             V22
             V23
             V24
             V25
             V26
             V27
             V28
             V29
             V30
             V31
             V32
             V33
             V34
             V35)
         (ERGO-IGNORE-IF-UNUSED V0
                                V1
                                V2
                                V3
                                V4
                                V5
                                V6
                                V7
                                V8
                                V9
                                V10
                                V11
                                V12
                                V13
                                V14
                                V15
                                V16
                                V17
                                V18
                                V19
                                V20
                                V21
                                V22
                                V23
                                V24
                                V25
                                V26
                                V27
                                V28
                                V29
                                V30
                                V31
                                V32
                                V33
                                V34
                                V35)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::LABEL)))
             (PROGN (LAM ((SBST::LABEL EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLEPLUS-PARSE ((SBST::!NUMBER! EPSILON))
                                      (LAM ((SBST::!NUMBER! EPSILON))
                                           (GOBBLE-TO-SLOT V5))
                                      V4
                                      V5
                                      'SBST::|,|)
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V1 1)
                    (AS-TO-SLOT V3
                                (MAKE-SB-TERM 'LABELS
                                              (DS-TEMP-REPT (CK-REPT-REF V4))))
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (OPT-PARSE ((SBST::CONST SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::CONST EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE ((SBST::!ID! SBST::=))
                                             (PROGN
                                               (LAM ((SBST::!ID! EPSILON))
                                                    (GOBBLE-TO-SLOT V11))
                                               (LAM ((SBST::= EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (CODE-TO-SLOT V12
                                                             (!CONSTANT! 0))
                                               (LAM ((SBST::|;| EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (AS-TO-SLOT V13
                                                           (MAKE-SB-TERM
                                                             'PASCAL::CONST-ENTRY
                                                             (LIST V11 V12)))
                                               (VALUE-TO-SLOT V10 V13))
                                             V9
                                             V10)
                                 (AS-TO-SLOT V8
                                             (MAKE-SB-TERM 'PASCAL::CONSTANTS
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V9)))))
                               V6)
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (OPT-PARSE ((SBST::TYPE SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::TYPE EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE ((SBST::!ID! SBST::=))
                                             (PROGN
                                               (LAM ((SBST::!ID! EPSILON))
                                                    (GOBBLE-TO-SLOT V19))
                                               (LAM ((SBST::= EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (CODE-TO-SLOT V20
                                                             (!MY-TYPE! 0))
                                               (LAM ((SBST::|;| EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (AS-TO-SLOT V21
                                                           (MAKE-SB-TERM
                                                             'PASCAL::TYPE-ENTRY
                                                             (LIST V19 V20)))
                                               (VALUE-TO-SLOT V18 V21))
                                             V17
                                             V18)
                                 (AS-TO-SLOT V16
                                             (MAKE-SB-TERM 'PASCAL::TYPES
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V17)))))
                               V14)
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (OPT-PARSE ((SBST::VAR SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::VAR EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE
                                   ((SBST::!ID! SBST::|,|
                                                SBST::|:|))
                                   (PROGN
                                     (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                                       (LAM
                                                         ((SBST::!ID! EPSILON))
                                                         (GOBBLE-TO-SLOT V28))
                                                       V27
                                                       V28
                                                       'SBST::|,|)
                                     (LAM ((SBST::|:| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V29
                                                   (!MY-TYPE! 0))
                                     (LAM ((SBST::|;| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (AS-TO-SLOT V30
                                                 (MAKE-SB-TERM
                                                   'PASCAL::VAR-ENTRY
                                                   (LIST
                                                     (MAKE-SB-TERM
                                                       'PASCAL::NAMES
                                                       (DS-TEMP-REPT
                                                         (CK-REPT-REF V27)))
                                                     V29)))
                                     (VALUE-TO-SLOT V26 V30))
                                   V25
                                   V26)
                                 (AS-TO-SLOT V24
                                             (MAKE-SB-TERM 'PASCAL::VARIABLES
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V25)))))
                               V22)
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (STAR-PARSE
                      ((SBST::PROCEDURE SBST::!ID!)
                        (SBST::FUNCTION SBST::!ID!))
                      (PROGN (CODE-TO-SLOT V33
                                           (!FUNCTION-LIST! 0))
                             (LAM ((SBST::|;| EPSILON))
                                  (GOBBLE-TOKEN))
                             (VALUE-TO-SLOT V32 V33))
                      V31
                      V32)
                    (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35
                                    (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           ((LA-MATCH ((SBST::CONST)))
             (PROGN (LAM ((SBST::CONST EPSILON))
                         (GOBBLE-TOKEN))
                    (PLUS-PARSE ((SBST::!ID! SBST::=))
                                (PROGN
                                  (LAM ((SBST::!ID! EPSILON))
                                       (GOBBLE-TO-SLOT V11))
                                  (LAM ((SBST::= EPSILON))
                                       (GOBBLE-TOKEN))
                                  (CODE-TO-SLOT V12
                                                (!CONSTANT! 0))
                                  (LAM ((SBST::|;| EPSILON))
                                       (GOBBLE-TOKEN))
                                  (AS-TO-SLOT V13
                                              (MAKE-SB-TERM
                                                'PASCAL::CONST-ENTRY
                                                (LIST V11 V12)))
                                  (VALUE-TO-SLOT V10 V13))
                                V9
                                V10)
                    (VALUE-TO-SLOT V6 1)
                    (AS-TO-SLOT V8
                                (MAKE-SB-TERM 'PASCAL::CONSTANTS
                                              (DS-TEMP-REPT (CK-REPT-REF V9))))
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (OPT-PARSE ((SBST::TYPE SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::TYPE EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE ((SBST::!ID! SBST::=))
                                             (PROGN
                                               (LAM ((SBST::!ID! EPSILON))
                                                    (GOBBLE-TO-SLOT V19))
                                               (LAM ((SBST::= EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (CODE-TO-SLOT V20
                                                             (!MY-TYPE! 0))
                                               (LAM ((SBST::|;| EPSILON))
                                                    (GOBBLE-TOKEN))
                                               (AS-TO-SLOT V21
                                                           (MAKE-SB-TERM
                                                             'PASCAL::TYPE-ENTRY
                                                             (LIST V19 V20)))
                                               (VALUE-TO-SLOT V18 V21))
                                             V17
                                             V18)
                                 (AS-TO-SLOT V16
                                             (MAKE-SB-TERM 'PASCAL::TYPES
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V17)))))
                               V14)
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (OPT-PARSE ((SBST::VAR SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::VAR EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE
                                   ((SBST::!ID! SBST::|,| SBST::|:|))
                                   (PROGN
                                     (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                                       (LAM
                                                         ((SBST::!ID! EPSILON))
                                                         (GOBBLE-TO-SLOT V28))
                                                       V27
                                                       V28
                                                       'SBST::|,|)
                                     (LAM ((SBST::|:| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V29
                                                   (!MY-TYPE! 0))
                                     (LAM ((SBST::|;| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (AS-TO-SLOT V30
                                                 (MAKE-SB-TERM
                                                   'PASCAL::VAR-ENTRY
                                                   (LIST
                                                     (MAKE-SB-TERM
                                                       'PASCAL::NAMES
                                                       (DS-TEMP-REPT
                                                         (CK-REPT-REF V27)))
                                                     V29)))
                                     (VALUE-TO-SLOT V26 V30))
                                   V25
                                   V26)
                                 (AS-TO-SLOT V24
                                             (MAKE-SB-TERM 'PASCAL::VARIABLES
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V25)))))
                               V22)
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (STAR-PARSE
                      ((SBST::PROCEDURE SBST::!ID!)
                        (SBST::FUNCTION SBST::!ID!))
                      (PROGN (CODE-TO-SLOT V33
                                           (!FUNCTION-LIST! 0))
                             (LAM ((SBST::|;| EPSILON))
                                  (GOBBLE-TOKEN))
                             (VALUE-TO-SLOT V32 V33))
                      V31
                      V32)
                    (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35 (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           ((LA-MATCH ((SBST::TYPE)))
             (PROGN (LAM ((SBST::TYPE EPSILON))
                         (GOBBLE-TOKEN))
                    (PLUS-PARSE ((SBST::!ID! SBST::=))
                                (PROGN
                                  (LAM ((SBST::!ID! EPSILON))
                                       (GOBBLE-TO-SLOT V19))
                                  (LAM ((SBST::= EPSILON))
                                       (GOBBLE-TOKEN))
                                  (CODE-TO-SLOT V20
                                                (!MY-TYPE! 0))
                                  (LAM ((SBST::|;| EPSILON))
                                       (GOBBLE-TOKEN))
                                  (AS-TO-SLOT V21
                                              (MAKE-SB-TERM 'PASCAL::TYPE-ENTRY
                                                            (LIST V19 V20)))
                                  (VALUE-TO-SLOT V18 V21))
                                V17
                                V18)
                    (VALUE-TO-SLOT V14 1)
                    (AS-TO-SLOT V16
                                (MAKE-SB-TERM 'PASCAL::TYPES
                                              (DS-TEMP-REPT (CK-REPT-REF V17))))
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (OPT-PARSE ((SBST::VAR SBST::!ID!))
                               (PROGN
                                 (LAM ((SBST::VAR EPSILON))
                                      (GOBBLE-TOKEN))
                                 (PLUS-PARSE
                                   ((SBST::!ID! SBST::|,| SBST::|:|))
                                   (PROGN
                                     (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                                       (LAM
                                                         ((SBST::!ID! EPSILON))
                                                         (GOBBLE-TO-SLOT V28))
                                                       V27
                                                       V28
                                                       'SBST::|,|)
                                     (LAM ((SBST::|:| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V29
                                                   (!MY-TYPE! 0))
                                     (LAM ((SBST::|;| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (AS-TO-SLOT V30
                                                 (MAKE-SB-TERM
                                                   'PASCAL::VAR-ENTRY
                                                   (LIST
                                                     (MAKE-SB-TERM
                                                       'PASCAL::NAMES
                                                       (DS-TEMP-REPT
                                                         (CK-REPT-REF V27)))
                                                     V29)))
                                     (VALUE-TO-SLOT V26 V30))
                                   V25
                                   V26)
                                 (AS-TO-SLOT V24
                                             (MAKE-SB-TERM 'PASCAL::VARIABLES
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V25)))))
                               V22)
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (STAR-PARSE
                      ((SBST::PROCEDURE SBST::!ID!)
                        (SBST::FUNCTION SBST::!ID!))
                      (PROGN (CODE-TO-SLOT V33
                                           (!FUNCTION-LIST! 0))
                             (LAM ((SBST::|;| EPSILON))
                                  (GOBBLE-TOKEN))
                             (VALUE-TO-SLOT V32 V33))
                      V31
                      V32)
                    (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35 (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V6 0)
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           ((LA-MATCH ((SBST::VAR)))
             (PROGN (LAM ((SBST::VAR EPSILON))
                         (GOBBLE-TOKEN))
                    (PLUS-PARSE ((SBST::!ID! SBST::|,| SBST::|:|))
                                (PROGN
                                  (DOUBLEPLUS-PARSE ((SBST::!ID! EPSILON))
                                                    (LAM
                                                      ((SBST::!ID! EPSILON))
                                                      (GOBBLE-TO-SLOT V28))
                                                    V27
                                                    V28
                                                    'SBST::|,|)
                                  (LAM ((SBST::|:| EPSILON))
                                       (GOBBLE-TOKEN))
                                  (CODE-TO-SLOT V29
                                                (!MY-TYPE! 0))
                                  (LAM ((SBST::|;| EPSILON))
                                       (GOBBLE-TOKEN))
                                  (AS-TO-SLOT V30
                                              (MAKE-SB-TERM 'PASCAL::VAR-ENTRY
                                                            (LIST
                                                              (MAKE-SB-TERM
                                                                'PASCAL::NAMES
                                                                (DS-TEMP-REPT
                                                                  (CK-REPT-REF
                                                                    V27)))
                                                              V29)))
                                  (VALUE-TO-SLOT V26 V30))
                                V25
                                V26)
                    (VALUE-TO-SLOT V22 1)
                    (AS-TO-SLOT V24
                                (MAKE-SB-TERM 'PASCAL::VARIABLES
                                              (DS-TEMP-REPT (CK-REPT-REF V25))))
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (STAR-PARSE
                      ((SBST::PROCEDURE SBST::!ID!)
                        (SBST::FUNCTION SBST::!ID!))
                      (PROGN (CODE-TO-SLOT V33
                                           (!FUNCTION-LIST! 0))
                             (LAM ((SBST::|;| EPSILON))
                                  (GOBBLE-TOKEN))
                             (VALUE-TO-SLOT V32 V33))
                      V31
                      V32)
                    (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35 (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V14 0)
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (VALUE-TO-SLOT V6 0)
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           ((LA-MATCH ((SBST::BEGIN)))
             (PROGN (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35 (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V22 0)
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (VALUE-TO-SLOT V14 0)
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (VALUE-TO-SLOT V6 0)
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           ((LA-MATCH ((SBST::FUNCTION) (SBST::PROCEDURE)))
             (PROGN (CODE-TO-SLOT V33 (!FUNCTION-LIST! 0))
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V32 V33)
                    (CONS-TO-SLOT V31 V32)
                    (GEN-STAR-PARSE
                      ((SBST::PROCEDURE SBST::!ID!)
                        (SBST::FUNCTION SBST::!ID!))
                      (PROGN (CODE-TO-SLOT V33
                                           (!FUNCTION-LIST! 0))
                             (LAM ((SBST::|;| EPSILON))
                                  (GOBBLE-TOKEN))
                             (VALUE-TO-SLOT V32 V33))
                      V31
                      V32)
                    (LAM ((SBST::BEGIN EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE
                      ((SBST::GOTO SBST::!NUMBER!) (SBST::WITH SBST::!ID!)
                                                   (SBST::CASE SBST::!NUMBER!
                                                               SBST::-
                                                               SBST::+
                                                               SBST::NOT
                                                               SBST::|(|
                                                               SBST::!ID!)
                                                   (SBST::FOR SBST::!ID!)
                                                   (SBST::WHILE SBST::!NUMBER!
                                                                SBST::-
                                                                SBST::+
                                                                SBST::NOT
                                                                SBST::|(|
                                                                SBST::!ID!)
                                                   (SBST::REPEAT SBST::UNTIL
                                                                 SBST::!NUMBER!
                                                                 SBST::BEGIN
                                                                 SBST::IF
                                                                 SBST::REPEAT
                                                                 SBST::WHILE
                                                                 SBST::FOR
                                                                 SBST::CASE
                                                                 SBST::WITH
                                                                 SBST::GOTO
                                                                 SBST::!ID!)
                                                   (SBST::IF SBST::!NUMBER!
                                                             SBST::-
                                                             SBST::+
                                                             SBST::NOT
                                                             SBST::|(|
                                                             SBST::!ID!)
                                                   (SBST::BEGIN SBST::END
                                                                SBST::!NUMBER!
                                                                SBST::BEGIN
                                                                SBST::IF
                                                                SBST::REPEAT
                                                                SBST::WHILE
                                                                SBST::FOR
                                                                SBST::CASE
                                                                SBST::WITH
                                                                SBST::GOTO
                                                                SBST::!ID!)
                                                   (SBST::!ID! SBST::|:=|
                                                               SBST::^
                                                               SBST::|.|
                                                               SBST::[
                                                               SBST::|(|
                                                               EPSILON)
                                                   (SBST::!NUMBER! SBST::|:|))
                      (CODE-TO-SLOT V35 (!STATEMENT! 0))
                      V34
                      V35
                      'SBST::|;|)
                    (LAM ((SBST::END EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V22 0)
                    (AS-TO-SLOT V23
                                (IF (= V22 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-4
                                                  NIL)
                                    V24))
                    (VALUE-TO-SLOT V14 0)
                    (AS-TO-SLOT V15
                                (IF (= V14 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-3
                                                  NIL)
                                    V16))
                    (VALUE-TO-SLOT V6 0)
                    (AS-TO-SLOT V7
                                (IF (= V6 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-2
                                                  NIL)
                                    V8))
                    (VALUE-TO-SLOT V1 0)
                    (AS-TO-SLOT V2
                                (IF (= V1 0)
                                    (MAKE-SB-TERM 'PASCAL::BLOCK-NULL-1
                                                  NIL)
                                    V3))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'BLOCK
                                              (LIST V2
                                                    V7
                                                    V15
                                                    V23
                                                    (MAKE-SB-TERM 'PASCAL::FUNS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V31)))
                                                    (MAKE-SB-TERM
                                                      'PASCAL::STATS
                                                      (DS-TEMP-REPT
                                                        (CK-REPT-REF V34))))))))
           (T
             (INITIAL-ERROR
               '((SBST::FUNCTION SBST::!ID!) (SBST::PROCEDURE SBST::!ID!)
                 (SBST::BEGIN SBST::END SBST::GOTO SBST::WITH SBST::CASE
                  SBST::FOR SBST::WHILE SBST::REPEAT SBST::IF SBST::BEGIN
                  SBST::!ID! SBST::!NUMBER!)
                 (SBST::VAR SBST::!ID!) (SBST::TYPE SBST::!ID!)
                 (SBST::CONST SBST::!ID!) (SBST::LABEL SBST::!NUMBER!)))))
         NIL
         V0))


(DEFUN !PROGRAM!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::PROGRAM)))
             (PROGN (LAM ((SBST::PROGRAM EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (LAM ((SBST::|(| EPSILON))
                         (GOBBLE-TOKEN))
                    (DOUBLESTAR-PARSE ((SBST::!ID! EPSILON))
                                      (LAM ((SBST::!ID! EPSILON))
                                           (GOBBLE-TO-SLOT V3))
                                      V2
                                      V3
                                      'SBST::|,|)
                    (LAM ((SBST::|)| EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::|;| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V4 (!BLOCK! 0))
                    (LAM ((SBST::PROGRAM EPSILON))
                         (GOBBLE-TOKEN))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'PASCAL::PROGRAM
                                              (LIST V1
                                                    (MAKE-SB-TERM 'PASCAL::ARGS
                                                                  (DS-TEMP-REPT
                                                                    (CK-REPT-REF
                                                                      V2)))
                                                    V4)))))
           (T (INITIAL-ERROR '((SBST::PROGRAM SBST::!ID!)))))
         NIL
         V0))

