;;; -*- Mode: Lisp; Package: PASCAL -*-
(in-package "PASCAL")  ;; creates package for abstract syntax. 

(in-package "PASCAL")  ;; enters package for generated code.  

(use-package '("ERGOLISP" "OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER *PASCAL-OPSIG-TABLE*
              (MAKE-OPSIG-TABLE
                '((#^LE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^FUNCTION-CALL
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXPS)) :-> #t(:SORT EXP)))
                  (#^FUNCT-DEF
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT PARAMS) #t(:SORT SB-RUNTIME::ID) #t(:SORT BLOCK)) :-> #t(:SORT FUNCTION-LIST)))
                  (#^LT
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^NOT . #@("pascal" := (#t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^WHILE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^ENUMERATED-TYPE
                   . #@("pascal" := (#t(:LIST (:SORT SB-RUNTIME::ID))) :-> #t(:SORT SIMPLE-TYPE)))
                  (#^ARRAY-REF
                   . #@("pascal" := (#t(:LIST (:SORT EXP))) :-> #t(:SORT VARIABLE-OPT-0)))
                  (#^UNI-PLUS . #@("pascal" := (#t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^POINTER-REF
                   . #@("pascal" := NIL :-> #t(:SORT VARIABLE-OPT-0)))
                  (#^PACKED-ARRAY
                   . #@("pascal" := (#t(:SORT SUBSCRIPTS) #t(:SORT MY-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^WITH
                   . #@("pascal" := (#t(:SORT VARS) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^TYPE-ENTRY
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT MY-TYPE)) :-> #t(:SORT TYPE-ENTRY)))
                  (#^POINTER
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT MY-TYPE)))
                  (#^MOD
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^ASSIGN
                   . #@("pascal" := (#t(:SORT VARIABLE) #t(:SORT EXP)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^DIV
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^FUNCT-PARAM
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT PARAMS) #t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT HEADING)))
                  (#^BLOCK
                   . #@("pascal" := (#t(:SORT BLOCK-OPT-0) #t(:SORT BLOCK-OPT-1) #t(:SORT BLOCK-OPT-2) #t(:SORT BLOCK-OPT-3) #t(:SORT FUNS) #t(:SORT STATS)) :-> #t(:SORT BLOCK)))
                  (#^FUNS
                   . #@("pascal" := (#t(:LIST (:SORT FUNCTION-LIST))) :-> #t(:SORT FUNS)))
                  (#^CASE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT CHOICES)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^NAMES
                   . #@("pascal" := (#t(:LIST (:SORT SB-RUNTIME::ID))) :-> #t(:SORT NAMES)))
                  (#^PROC-PARAM
                   . #@("pascal" := (#t(:SORT HEADING)) :-> #t(:SORT PARAMETER-TYPE)))
                  (#^PLUS
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^IF-THEN-ELSE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT STATEMENT) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^SET
                   . #@("pascal" := (#t(:SORT SIMPLE-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^TYPES
                   . #@("pascal" := (#t(:LIST (:OP #^TYPE-ENTRY))) :-> #t(:SORT BLOCK-OPT-2)))
                  (#^BRANCH
                   . #@("pascal" := (#t(:SORT CONSTANTS) #t(:SORT FIELD-LIST-OPT-4)) :-> #t(:SORT BRANCH)))
                  (#^FILE
                   . #@("pascal" := (#t(:SORT MY-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^VARS
                   . #@("pascal" := (#t(:LIST (:SORT VARIABLE))) :-> #t(:SORT VARS)))
                  (#^PROGRAM
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT ARGS) #t(:SORT BLOCK)) :-> #t(:SORT PROGRAM)))
                  (#^VARIABLE-NULL-1
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT VARIABLE-OPT-0)))
                  (#^NULL . #@("pascal" := NIL :-> #t(:SORT CONSTANT)))
                  (#^CHOICES
                   . #@("pascal" := (#t(:LIST (:OP #^CASE-ENTRY))) :-> #t(:SORT CHOICES)))
                  (#^FIELD-LIST-NULL-5
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT FIELD-LIST-OPT-4)))
                  (#^LABELS
                   . #@("pascal" := (#t(:LIST (:SORT NUMBER))) :-> #t(:SORT BLOCK-OPT-0)))
                  (#^IF-THEN
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^MY-TYPE-NULL-1
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT MY-TYPE-OPT-0)))
                  (#^FLIST
                   . #@("pascal" := (#t(:SORT FIELD-LIST)) :-> #t(:SORT MY-TYPE-OPT-0)))
                  (#^BRANCHES
                   . #@("pascal" := (#t(:LIST (:OP #^BRANCH))) :-> #t(:SORT BRANCHES)))
                  (#^FIELD-LIST-NULL-2
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT FIELD-LIST-OPT-1)))
                  (#^FIELD-LIST-NULL-1
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT FIELD-LIST-OPT-0)))
                  (#^FIELD-LIST-NULL-4
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT FIELD-LIST-OPT-3)))
                  (#^FIELD-LIST-NULL-3
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT FIELD-LIST-OPT-2)))
                  (#^IDS
                   . #@("pascal" := (#t(:LIST (:SORT SB-RUNTIME::ID))) :-> #t(:SORT IDS)))
                  (#^VARIANT-PART
                   . #@("pascal" := (#t(:SORT FIELD-LIST-OPT-3) #t(:SORT SB-RUNTIME::ID) #t(:SORT BRANCHES)) :-> #t(:SORT FIELD-LIST)))
                  (#^SUBSCRIPTS
                   . #@("pascal" := (#t(:LIST (:SORT SIMPLE-TYPE))) :-> #t(:SORT SUBSCRIPTS)))
                  (#^OR
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^BLOCK-NULL-4
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT BLOCK-OPT-3)))
                  (#^BLOCK-NULL-2
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT BLOCK-OPT-1)))
                  (#^BLOCK-NULL-3
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT BLOCK-OPT-2)))
                  (#^BLOCK-NULL-1
                   . #@("pascal" := (#t(:NULL)) :-> #t(:SORT BLOCK-OPT-0)))
                  (#^NUMBER-STAT
                   . #@("pascal" := (#t(:SORT NUMBER) #t(:SORT REAL-STATEMENT)) :-> #t(:SORT STATEMENT)))
                  (#^VAL
                   . #@("pascal" := (#t(:SORT IDS) #t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT PARAMETER-TYPE)))
                  (#^UNI-MINUS
                   . #@("pascal" := (#t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^PACKED-RECORD
                   . #@("pascal" := (#t(:SORT MY-TYPE-OPT-0)) :-> #t(:SORT MY-TYPE)))
                  (#^CONDS
                   . #@("pascal" := (#t(:LIST (:SORT CONSTANT))) :-> #t(:SORT CONDS)))
                  (#^FOR-DOWNTO
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXP) #t(:SORT EXP) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^PACKED-SET
                   . #@("pascal" := (#t(:SORT SIMPLE-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^PROC-CALL
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXPS)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^EXPS
                   . #@("pascal" := (#t(:LIST (:SORT EXP))) :-> #t(:SORT EXPS)))
                  (#^IN
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^VARIABLES
                   . #@("pascal" := (#t(:LIST (:OP #^VAR-ENTRY))) :-> #t(:SORT BLOCK-OPT-3)))
                  (#^PROC-DEF
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT PARAMS) #t(:SORT BLOCK)) :-> #t(:SORT FUNCTION-LIST)))
                  (#^RECORD
                   . #@("pascal" := (#t(:SORT MY-TYPE-OPT-0)) :-> #t(:SORT MY-TYPE)))
                  (#^GE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^AND
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^REF
                   . #@("pascal" := (#t(:SORT IDS) #t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT PARAMETER-TYPE)))
                  (#^ARGS
                   . #@("pascal" := (#t(:LIST (:SORT SB-RUNTIME::ID))) :-> #t(:SORT ARGS)))
                  (#^COMPLEX-REF
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT VARIABLE-OPT-0)) :-> #t(:SORT VARIABLE)))
                  (#^REPEAT
                   . #@("pascal" := (#t(:SORT STATS) #t(:SORT EXP)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^GT
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^NE
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^FIELDS
                   . #@("pascal" := (#t(:SORT IDS) #t(:SORT MY-TYPE)) :-> #t(:SORT FIELDS)))
                  (#^STATS
                   . #@("pascal" := (#t(:LIST (:SORT STATEMENT))) :-> #t(:SORT STATS)))
                  (#^GOTO
                   . #@("pascal" := (#t(:SORT NUMBER)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^ARRAY
                   . #@("pascal" := (#t(:SORT SUBSCRIPTS) #t(:SORT MY-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^FOR-TO
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXP) #t(:SORT EXP) #t(:SORT STATEMENT)) :-> #t(:SORT REAL-STATEMENT)))
                  (#^CONSTANTS
                   . #@("pascal" := (#t(:LIST (:SORT CONSTANT))) :-> #t(:SORT CONSTANTS)))
                  (#^TIMES
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^CASE-ENTRY
                   . #@("pascal" := (#t(:SORT CONDS) #t(:SORT STATEMENT)) :-> #t(:SORT CASE-ENTRY)))
                  (#^VAR-ENTRY
                   . #@("pascal" := (#t(:SORT NAMES) #t(:SORT MY-TYPE)) :-> #t(:SORT VAR-ENTRY)))
                  (#^PACKED-FILE
                   . #@("pascal" := (#t(:SORT MY-TYPE)) :-> #t(:SORT MY-TYPE)))
                  (#^EQ
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^FIELD-LIST
                   . #@("pascal" := (#t(:SORT FIELDS) #t(:SORT FIELD-LIST-OPT-0)) :-> #t(:SORT FIELD-LIST)))
                  (#^PARAMS
                   . #@("pascal" := (#t(:LIST (:SORT PARAMETER-TYPE))) :-> #t(:SORT PARAMS)))
                  (#^RANGE
                   . #@("pascal" := (#t(:SORT CONSTANT) #t(:SORT CONSTANT)) :-> #t(:SORT SIMPLE-TYPE)))
                  (#^REC-FIELD
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT VARIABLE-OPT-0)))
                  (#^CONST-ENTRY
                   . #@("pascal" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT CONSTANT)) :-> #t(:SORT CONST-ENTRY)))
                  (#^MINUS
                   . #@("pascal" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP))))))


(DEFPARAMETER *PASCAL-SORT-TABLE*
              (MAKE-SORT-TABLE
                '((#t(:SORT SUBSCRIPTS) . #t(:OP #^SUBSCRIPTS))
                  (#t(:SORT VARS) . #t(:OP #^VARS))
                  (#t(:SORT CONSTANT)
                   . #t(:UNION (:OP #^NULL) (:SORT STRING) (:SORT SB-RUNTIME::ID) (:SORT NUMBER)))
                  (#t(:SORT STATEMENT)
                   . #t(:UNION (:OP #^NUMBER-STAT) (:SORT REAL-STATEMENT)))
                  (#t(:SORT NAMES) . #t(:OP #^NAMES))
                  (#t(:SORT BRANCH) . #t(:OP #^BRANCH))
                  (#t(:SORT TYPE-ENTRY) . #t(:OP #^TYPE-ENTRY))
                  (#t(:SORT CASE-ENTRY) . #t(:OP #^CASE-ENTRY))
                  (#t(:SORT MY-TYPE-OPT-0)
                   . #t(:UNION (:OP #^FLIST) (:OP #^MY-TYPE-NULL-1)))
                  (#t(:SORT FIELDS) . #t(:OP #^FIELDS))
                  (#t(:SORT VAR-ENTRY) . #t(:OP #^VAR-ENTRY))
                  (#t(:SORT CONST-ENTRY) . #t(:OP #^CONST-ENTRY))
                  (#t(:SORT CONSTANTS) . #t(:OP #^CONSTANTS))
                  (#t(:SORT BRANCHES) . #t(:OP #^BRANCHES))
                  (#t(:SORT FIELD-LIST)
                   . #t(:UNION (:OP #^VARIANT-PART) (:OP #^FIELD-LIST)))
                  (#t(:SORT BLOCK) . #t(:OP #^BLOCK))
                  (#t(:SORT ARGS) . #t(:OP #^ARGS))
                  (#t(:SORT MY-TYPE)
                   . #t(:UNION (:OP #^PACKED-FILE) (:OP #^FILE) (:OP #^PACKED-RECORD) (:OP #^RECORD) (:OP #^PACKED-ARRAY) (:OP #^ARRAY) (:OP #^PACKED-SET) (:OP #^SET) (:OP #^POINTER) (:SORT SIMPLE-TYPE)))
                  (#t(:SORT HEADING)
                   . #t(:UNION (:OP #^FUNCT-PARAM) (:OP #^PROC-PARAM)))
                  (#t(:SORT FUNS) . #t(:OP #^FUNS))
                  (#t(:SORT BLOCK-OPT-0)
                   . #t(:UNION (:OP #^LABELS) (:OP #^BLOCK-NULL-1)))
                  (#t(:SORT BLOCK-OPT-1)
                   . #t(:UNION (:OP #^CONSTANTS) (:OP #^BLOCK-NULL-2)))
                  (#t(:SORT BLOCK-OPT-2)
                   . #t(:UNION (:OP #^TYPES) (:OP #^BLOCK-NULL-3)))
                  (#t(:SORT BLOCK-OPT-3)
                   . #t(:UNION (:OP #^VARIABLES) (:OP #^BLOCK-NULL-4)))
                  (#t(:SORT CHOICES) . #t(:OP #^CHOICES))
                  (#t(:SORT REAL-STATEMENT)
                   . #t(:UNION (:OP #^GOTO) (:OP #^WITH) (:OP #^CASE) (:OP #^FOR-DOWNTO) (:OP #^FOR-TO) (:OP #^WHILE) (:OP #^REPEAT) (:OP #^IF-THEN-ELSE) (:OP #^IF-THEN) (:OP #^PROC-CALL) (:OP #^STATS) (:OP #^ASSIGN)))
                  (#t(:SORT PARAMETER-TYPE)
                   . #t(:UNION (:OP #^PROC-PARAM) (:OP #^REF) (:OP #^VAL)))
                  (#t(:SORT VARIABLE)
                   . #t(:UNION (:OP #^COMPLEX-REF) (:SORT SB-RUNTIME::ID)))
                  (#t(:SORT FIELD-LIST-OPT-4)
                   . #t(:UNION (:SORT FIELD-LIST) (:OP #^FIELD-LIST-NULL-5)))
                  (#t(:SORT EXP)
                   . #t(:UNION (:OP #^FUNCTION-CALL) (:OP #^NOT) (:OP #^UNI-PLUS) (:OP #^UNI-MINUS) (:OP #^DIV) (:OP #^MOD) (:OP #^TIMES) (:OP #^AND) (:OP #^OR) (:OP #^MINUS) (:OP #^PLUS) (:OP #^IN) (:OP #^NE) (:OP #^GE) (:OP #^GT) (:OP #^EQ) (:OP #^LT) (:OP #^LE) (:SORT NUMBER) (:SORT VARIABLE)))
                  (#t(:SORT IDS) . #t(:OP #^IDS))
                  (#t(:SORT STATS) . #t(:OP #^STATS))
                  (#t(:SORT VARIABLE-OPT-0)
                   . #t(:UNION (:OP #^POINTER-REF) (:OP #^REC-FIELD) (:OP #^ARRAY-REF) (:OP #^VARIABLE-NULL-1)))
                  (#t(:SORT CONDS) . #t(:OP #^CONDS))
                  (#t(:SORT FIELD-LIST-OPT-3)
                   . #t(:UNION (:SORT SB-RUNTIME::ID) (:OP #^FIELD-LIST-NULL-4)))
                  (#t(:SORT FIELD-LIST-OPT-2)
                   . #t(:UNION (:SORT FIELD-LIST) (:OP #^FIELD-LIST-NULL-3)))
                  (#t(:SORT FIELD-LIST-OPT-1)
                   . #t(:UNION (:SORT SB-RUNTIME::ID) (:OP #^FIELD-LIST-NULL-2)))
                  (#t(:SORT FIELD-LIST-OPT-0)
                   . #t(:UNION (:OP #^VARIANT-PART) (:OP #^FIELD-LIST-NULL-1)))
                  (#t(:SORT FUNCTION-LIST)
                   . #t(:UNION (:OP #^FUNCT-DEF) (:OP #^PROC-DEF)))
                  (#t(:SORT SIMPLE-TYPE)
                   . #t(:UNION (:OP #^RANGE) (:OP #^ENUMERATED-TYPE) (:SORT SB-RUNTIME::ID)))
                  (#t(:SORT EXPS) . #t(:OP #^EXPS))
                  (#t(:SORT PARAMS) . #t(:OP #^PARAMS))
                  (#t(:SORT PROGRAM) . #t(:OP #^PROGRAM)))))

(lang:lang-define 
:name "pascal"
:conc-name "pascal"
:code-package "PASCAL"
:abs-syn-package "PASCAL"
:use-packages '("ERGOLISP" "OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 'T
:parse-routine-name 'PASCAL::PASCAL-PARSE
:unparse-routine-name 'PASCAL::PASCAL-UNPARSE
:win-unparse-routine-name 'PASCAL::PASCAL-WIN-UNPARSE
:sort-table-name 'PASCAL::*PASCAL-SORT-TABLE*
:opsig-table-name 'PASCAL::*PASCAL-OPSIG-TABLE*
)

